@@ -6,10 +6,10 @@
 //! Encoding <-> RbEncoding <-> Index
 //!       |______________________^
 //! ```
-//! Many functions that require an encoding take thier arguments as
+//! Many functions that require an encoding take their arguments as
 //! `Into<RbEncoding>` or `Into<Index>` to ease working with the different
 //! types. The type specified for the `Into` conversion hints at the type the
-//! function nativly works with, and thus will avoid any conversion cost.
+//! function natively works with, and thus will avoid any conversion cost.
 //!
 //! [`Encoding`] and [`RbEncoding`] both implement [`TryConvert`] and
 //! [`IntoValue`] so can be used as parameters and return values in functions
@@ -113,9 +113,10 @@ impl Encoding {
     /// Panics if called from a non-Ruby thread. See
     /// [`Ruby::enc_default_external`] for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::enc_default_external` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn default_external() -> Self {
         get_ruby!().enc_default_external()
@@ -131,9 +132,10 @@ impl Encoding {
     /// Panics if called from a non-Ruby thread. See
     /// [`Ruby::enc_default_internal`] for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::enc_default_internal` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn default_internal() -> Option<Self> {
         get_ruby!().enc_default_internal()
@@ -312,7 +314,7 @@ impl Ruby {
 pub struct RbEncoding(NonNull<rb_encoding>);
 
 impl RbEncoding {
-    fn new(inner: *mut rb_encoding) -> Option<Self> {
+    pub(crate) fn new(inner: *mut rb_encoding) -> Option<Self> {
         NonNull::new(inner).map(Self)
     }
 
@@ -323,9 +325,10 @@ impl RbEncoding {
     /// Panics if called from a non-Ruby thread. See
     /// [`Ruby::ascii8bit_encoding`] for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::ascii8bit_encoding` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn ascii8bit() -> Self {
         get_ruby!().ascii8bit_encoding()
@@ -338,9 +341,10 @@ impl RbEncoding {
     /// Panics if called from a non-Ruby thread. See [`Ruby::utf8_encoding`]
     /// for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::utf8_encoding` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn utf8() -> Self {
         get_ruby!().utf8_encoding()
@@ -353,9 +357,10 @@ impl RbEncoding {
     /// Panics if called from a non-Ruby thread. See [`Ruby::usascii_encoding`]
     /// for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::usascii_encoding` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn usascii() -> Self {
         get_ruby!().usascii_encoding()
@@ -371,9 +376,10 @@ impl RbEncoding {
     /// Panics if called from a non-Ruby thread. See [`Ruby::locale_encoding`]
     /// for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::locale_encoding` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn locale() -> Self {
         get_ruby!().locale_encoding()
@@ -389,9 +395,10 @@ impl RbEncoding {
     /// Panics if called from a non-Ruby thread. See
     /// [`Ruby::filesystem_encoding`] for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::filesystem_encoding` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn filesystem() -> Self {
         get_ruby!().filesystem_encoding()
@@ -407,9 +414,10 @@ impl RbEncoding {
     /// Panics if called from a non-Ruby thread. See
     /// [`Ruby::default_external_encoding`] for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::default_external_encoding` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn default_external() -> Self {
         get_ruby!().default_external_encoding()
@@ -425,9 +433,10 @@ impl RbEncoding {
     /// Panics if called from a non-Ruby thread. See
     /// [`Ruby::default_internal_encoding`] for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::default_internal_encoding` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn default_internal() -> Option<Self> {
         get_ruby!().default_internal_encoding()
@@ -443,6 +452,7 @@ impl RbEncoding {
     /// # Examples
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::encoding::RbEncoding;
     /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
@@ -450,9 +460,10 @@ impl RbEncoding {
     /// assert_eq!(RbEncoding::find("BINARY").unwrap().name(), "ASCII-8BIT");
     /// ```
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::find_encoding` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn find(name: &str) -> Option<Self> {
         get_ruby!().find_encoding(name)
@@ -467,11 +478,15 @@ impl RbEncoding {
     /// # Examples
     ///
     /// ```
-    /// use magnus::encoding::RbEncoding;
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, Ruby};
     ///
-    /// assert_eq!(RbEncoding::utf8().name(), "UTF-8");
-    /// assert_eq!(RbEncoding::find("UTF-16").unwrap().name(), "UTF-16");
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     assert_eq!(ruby.utf8_encoding().name(), "UTF-8");
+    ///     assert_eq!(ruby.find_encoding("UTF-16").unwrap().name(), "UTF-16");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// # Panics
@@ -488,11 +503,15 @@ impl RbEncoding {
     /// # Examples
     ///
     /// ```
-    /// use magnus::encoding::RbEncoding;
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, Ruby};
     ///
-    /// assert_eq!(RbEncoding::usascii().mbminlen(), 1);
-    /// assert_eq!(RbEncoding::utf8().mbminlen(), 1);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     assert_eq!(ruby.usascii_encoding().mbminlen(), 1);
+    ///     assert_eq!(ruby.utf8_encoding().mbminlen(), 1);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn mbminlen(&self) -> usize {
         unsafe { self.0.as_ref().min_enc_len as usize }
@@ -504,11 +523,15 @@ impl RbEncoding {
     /// # Examples
     ///
     /// ```
-    /// use magnus::encoding::RbEncoding;
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, Ruby};
     ///
-    /// assert_eq!(RbEncoding::usascii().mbmaxlen(), 1);
-    /// assert_eq!(RbEncoding::utf8().mbmaxlen(), 4);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     assert_eq!(ruby.usascii_encoding().mbmaxlen(), 1);
+    ///     assert_eq!(ruby.utf8_encoding().mbmaxlen(), 4);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn mbmaxlen(&self) -> usize {
         unsafe { self.0.as_ref().max_enc_len as usize }
@@ -530,26 +553,30 @@ impl RbEncoding {
     /// ```
     /// use magnus::{
     ///     encoding::{EncodingCapable, RbEncoding},
-    ///     RString,
+    ///     Error, Ruby,
     /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
-    /// let s = RString::new("🦀 café");
-    /// let encoding: RbEncoding = s.enc_get().into();
-    /// let mut chars = 0;
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.str_new("🦀 café");
+    ///     let encoding: RbEncoding = s.enc_get().into();
+    ///     let mut chars = 0;
     ///
-    /// unsafe {
-    ///     let mut bytes = s.as_slice();
-    ///     assert_eq!(bytes.len(), 10);
+    ///     unsafe {
+    ///         let mut bytes = s.as_slice();
+    ///         assert_eq!(bytes.len(), 10);
     ///
-    ///     while !bytes.is_empty() {
-    ///         chars += 1;
-    ///         let len = encoding.mbclen(bytes);
-    ///         bytes = &bytes[len..];
+    ///         while !bytes.is_empty() {
+    ///             chars += 1;
+    ///             let len = encoding.mbclen(bytes);
+    ///             bytes = &bytes[len..];
+    ///         }
     ///     }
-    /// }
     ///
-    /// assert_eq!(chars, 6);
+    ///     assert_eq!(chars, 6);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn mbclen(&self, slice: &[u8]) -> usize {
         let Range { start: p, end: e } = slice.as_ptr_range();
@@ -559,7 +586,7 @@ impl RbEncoding {
     /// Returns the number of bytes of the first character in `slice`.
     ///
     /// If the first byte of `slice` is mid way through a character this will
-    /// return the number of bytes until the next character boundry.
+    /// return the number of bytes until the next character boundary.
     ///
     /// If the slice ends before the last byte of the character this will
     /// return the theoretical number of bytes until the end of the character,
@@ -574,26 +601,30 @@ impl RbEncoding {
     /// ```
     /// use magnus::{
     ///     encoding::{EncodingCapable, RbEncoding},
-    ///     RString,
+    ///     Error, Ruby,
     /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
-    /// let s = RString::new("🦀 café");
-    /// let encoding: RbEncoding = s.enc_get().into();
-    /// let mut chars = 0;
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.str_new("🦀 café");
+    ///     let encoding: RbEncoding = s.enc_get().into();
+    ///     let mut chars = 0;
     ///
-    /// unsafe {
-    ///     let mut bytes = s.as_slice();
-    ///     assert_eq!(bytes.len(), 10);
+    ///     unsafe {
+    ///         let mut bytes = s.as_slice();
+    ///         assert_eq!(bytes.len(), 10);
     ///
-    ///     while !bytes.is_empty() {
-    ///         chars += 1;
-    ///         let len = encoding.fast_mbclen(bytes);
-    ///         bytes = &bytes[len..];
+    ///         while !bytes.is_empty() {
+    ///             chars += 1;
+    ///             let len = encoding.fast_mbclen(bytes);
+    ///             bytes = &bytes[len..];
+    ///         }
     ///     }
-    /// }
     ///
-    /// assert_eq!(chars, 6);
+    ///     assert_eq!(chars, 6);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn fast_mbclen(&self, slice: &[u8]) -> usize {
         let Range { start: p, end: e } = slice.as_ptr_range();
@@ -612,29 +643,33 @@ impl RbEncoding {
     /// ```
     /// use magnus::{
     ///     encoding::{EncodingCapable, MbcLen, RbEncoding},
-    ///     RString,
+    ///     Error, Ruby,
     /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
-    /// let s = RString::new("🦀 café");
-    /// let encoding: RbEncoding = s.enc_get().into();
-    /// let mut chars = 0;
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.str_new("🦀 café");
+    ///     let encoding: RbEncoding = s.enc_get().into();
+    ///     let mut chars = 0;
     ///
-    /// unsafe {
-    ///     let mut bytes = s.as_slice();
-    ///     assert_eq!(bytes.len(), 10);
+    ///     unsafe {
+    ///         let mut bytes = s.as_slice();
+    ///         assert_eq!(bytes.len(), 10);
     ///
-    ///     while !bytes.is_empty() {
-    ///         chars += 1;
-    ///         match encoding.precise_mbclen(bytes) {
-    ///             MbcLen::CharFound(len) => bytes = &bytes[len..],
-    ///             MbcLen::NeedMore(len) => panic!("Met end of string expecting {} bytes", len),
-    ///             MbcLen::Invalid => panic!("corrupted string"),
+    ///         while !bytes.is_empty() {
+    ///             chars += 1;
+    ///             match encoding.precise_mbclen(bytes) {
+    ///                 MbcLen::CharFound(len) => bytes = &bytes[len..],
+    ///                 MbcLen::NeedMore(len) => panic!("Met end of string expecting {} bytes", len),
+    ///                 MbcLen::Invalid => panic!("corrupted string"),
+    ///             }
     ///         }
     ///     }
-    /// }
     ///
-    /// assert_eq!(chars, 6);
+    ///     assert_eq!(chars, 6);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn precise_mbclen(&self, slice: &[u8]) -> MbcLen {
         let Range { start: p, end: e } = slice.as_ptr_range();
@@ -662,29 +697,33 @@ impl RbEncoding {
     /// ```
     /// use magnus::{
     ///     encoding::{EncodingCapable, RbEncoding},
-    ///     RString,
+    ///     Error, Ruby,
     /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
-    /// let s = RString::new("example");
-    /// let encoding: RbEncoding = s.enc_get().into();
-    /// let mut chars = Vec::new();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.str_new("example");
+    ///     let encoding: RbEncoding = s.enc_get().into();
+    ///     let mut chars = Vec::new();
     ///
-    /// unsafe {
-    ///     let mut bytes = s.as_slice();
+    ///     unsafe {
+    ///         let mut bytes = s.as_slice();
     ///
-    ///     while !bytes.is_empty() {
-    ///         match encoding.ascget(bytes) {
-    ///             Some((char, len)) => {
-    ///                 chars.push(char);
-    ///                 bytes = &bytes[len..];
+    ///         while !bytes.is_empty() {
+    ///             match encoding.ascget(bytes) {
+    ///                 Some((char, len)) => {
+    ///                     chars.push(char);
+    ///                     bytes = &bytes[len..];
+    ///                 }
+    ///                 None => panic!("string not ASCII"),
     ///             }
-    ///             None => panic!("string not ASCII"),
     ///         }
     ///     }
-    /// }
     ///
-    /// assert_eq!(chars, [101, 120, 97, 109, 112, 108, 101]);
+    ///     assert_eq!(chars, [101, 120, 97, 109, 112, 108, 101]);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn ascget(&self, slice: &[u8]) -> Option<(u8, usize)> {
         let Range { start: p, end: e } = slice.as_ptr_range();
@@ -700,7 +739,7 @@ impl RbEncoding {
         if len == 0 {
             panic!("{:?}", slice);
         }
-        (c > -1).then(|| (c as u8, len as usize))
+        (c > -1).then_some((c as u8, len as usize))
     }
 
     /// Returns the codepoint and length in bytes of the first character in
@@ -711,25 +750,29 @@ impl RbEncoding {
     /// ```
     /// use magnus::{
     ///     encoding::{EncodingCapable, RbEncoding},
-    ///     RString,
+    ///     Error, Ruby,
     /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
-    /// let s = RString::new("🦀 café");
-    /// let encoding: RbEncoding = s.enc_get().into();
-    /// let mut codepoints = Vec::new();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.str_new("🦀 café");
+    ///     let encoding: RbEncoding = s.enc_get().into();
+    ///     let mut codepoints = Vec::new();
     ///
-    /// unsafe {
-    ///     let mut bytes = s.as_slice();
+    ///     unsafe {
+    ///         let mut bytes = s.as_slice();
     ///
-    ///     while !bytes.is_empty() {
-    ///         let (codepoint, len) = encoding.codepoint_len(bytes).unwrap();
-    ///         codepoints.push(codepoint);
-    ///         bytes = &bytes[len..];
+    ///         while !bytes.is_empty() {
+    ///             let (codepoint, len) = encoding.codepoint_len(bytes)?;
+    ///             codepoints.push(codepoint);
+    ///             bytes = &bytes[len..];
+    ///         }
     ///     }
-    /// }
     ///
-    /// assert_eq!(codepoints, [129408, 32, 99, 97, 102, 233]);
+    ///     assert_eq!(codepoints, [129408, 32, 99, 97, 102, 233]);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn codepoint_len(&self, slice: &[u8]) -> Result<(u32, usize), Error> {
         let Range { start: p, end: e } = slice.as_ptr_range();
@@ -753,11 +796,15 @@ impl RbEncoding {
     /// # Examples
     ///
     /// ```
-    /// use magnus::encoding::RbEncoding;
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, Ruby};
     ///
-    /// assert_eq!(RbEncoding::utf8().codelen(97).unwrap(), 1);
-    /// assert_eq!(RbEncoding::utf8().codelen(129408).unwrap(), 4);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     assert_eq!(ruby.utf8_encoding().codelen(97)?, 1);
+    ///     assert_eq!(ruby.utf8_encoding().codelen(129408)?, 4);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn codelen(&self, code: u32) -> Result<usize, Error> {
         let handle = unsafe { Ruby::get_unchecked() };
@@ -780,21 +827,29 @@ impl RbEncoding {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{encoding::RbEncoding, eval};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{eval, Error, Ruby};
     ///
-    /// let c = RbEncoding::usascii().chr(97).unwrap();
-    /// let res: bool = eval!(r#"c == "a""#, c).unwrap();
-    /// assert!(res);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let c = ruby.usascii_encoding().chr(97)?;
+    ///     let res: bool = eval!(ruby, r#"c == "a""#, c)?;
+    ///     assert!(res);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{encoding::RbEncoding, eval};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{eval, Error, Ruby};
     ///
-    /// let c = RbEncoding::utf8().chr(129408).unwrap();
-    /// let res: bool = eval!(r#"c == "🦀""#, c).unwrap();
-    /// assert!(res);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let c = ruby.utf8_encoding().chr(129408)?;
+    ///     let res: bool = eval!(ruby, r#"c == "🦀""#, c)?;
+    ///     assert!(res);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn chr(&self, code: u32) -> Result<RString, Error> {
         protect(|| unsafe {
@@ -808,11 +863,15 @@ impl RbEncoding {
     /// # Examples
     ///
     /// ```
-    /// use magnus::encoding::RbEncoding;
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, Ruby};
     ///
-    /// assert!(RbEncoding::utf8().is_mbc_newline(&[10]));
-    /// assert!(!RbEncoding::utf8().is_mbc_newline(&[32]));
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     assert!(ruby.utf8_encoding().is_mbc_newline(&[10]));
+    ///     assert!(!ruby.utf8_encoding().is_mbc_newline(&[32]));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn is_mbc_newline(&self, slice: &[u8]) -> bool {
         let Range { start: p, end: e } = slice.as_ptr_range();
@@ -828,14 +887,18 @@ impl RbEncoding {
     /// # Examples
     ///
     /// ```
-    /// use magnus::encoding::{CType, RbEncoding};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{encoding::CType, Error, Ruby};
     ///
-    /// assert!(RbEncoding::utf8().is_code_ctype(9, CType::Space)); // "\t"
-    /// assert!(RbEncoding::utf8().is_code_ctype(32, CType::Space)); // " "
-    /// assert!(!RbEncoding::utf8().is_code_ctype(65, CType::Space)); // "A"
-    /// assert!(RbEncoding::utf8().is_code_ctype(65, CType::Alnum)); // "A"
-    /// assert!(RbEncoding::utf8().is_code_ctype(65, CType::Upper)); // "A"
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     assert!(ruby.utf8_encoding().is_code_ctype(9, CType::Space)); // "\t"
+    ///     assert!(ruby.utf8_encoding().is_code_ctype(32, CType::Space)); // " "
+    ///     assert!(!ruby.utf8_encoding().is_code_ctype(65, CType::Space)); // "A"
+    ///     assert!(ruby.utf8_encoding().is_code_ctype(65, CType::Alnum)); // "A"
+    ///     assert!(ruby.utf8_encoding().is_code_ctype(65, CType::Upper)); // "A"
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn is_code_ctype(&self, code: u32, ctype: CType) -> bool {
         unsafe { self.0.as_ref().is_code_ctype.unwrap()(code, ctype as _, self.as_ptr()) != 0 }
@@ -1006,9 +1069,10 @@ impl Index {
     /// Panics if called from a non-Ruby thread. See
     /// [`Ruby::ascii8bit_encindex`] for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::ascii8bit_encindex` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn ascii8bit() -> Self {
         get_ruby!().ascii8bit_encindex()
@@ -1021,9 +1085,10 @@ impl Index {
     /// Panics if called from a non-Ruby thread. See [`Ruby::utf8_encindex`]
     /// for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::utf8_encindex` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn utf8() -> Self {
         get_ruby!().utf8_encindex()
@@ -1036,9 +1101,10 @@ impl Index {
     /// Panics if called from a non-Ruby thread. See [`Ruby::usascii_encindex`]
     /// for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::usascii_encindex` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn usascii() -> Self {
         get_ruby!().usascii_encindex()
@@ -1054,9 +1120,10 @@ impl Index {
     /// Panics if called from a non-Ruby thread. See [`Ruby::locale_encindex`]
     /// for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::locale_encindex` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn locale() -> Self {
         get_ruby!().locale_encindex()
@@ -1072,9 +1139,10 @@ impl Index {
     /// Panics if called from a non-Ruby thread. See
     /// [`Ruby::filesystem_encindex`] for the non-panicking version.
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::filesystem_encindex` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn filesystem() -> Self {
         get_ruby!().filesystem_encindex()
@@ -1090,6 +1158,7 @@ impl Index {
     /// # Examples
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::encoding;
     /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
@@ -1098,9 +1167,10 @@ impl Index {
     /// assert!(encoding::Index::find("none").is_err());
     /// ```
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::find_encindex` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn find(name: &str) -> Result<Self, Error> {
         get_ruby!().find_encindex(name)
@@ -1153,13 +1223,14 @@ pub trait EncodingCapable: ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{
-    ///     encoding::{self, EncodingCapable},
-    ///     RString,
-    /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{encoding::EncodingCapable, Error, Ruby};
     ///
-    /// assert!(RString::new("example").enc_get() == encoding::Index::utf8());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     assert!(ruby.str_new("example").enc_get() == ruby.utf8_encindex());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn enc_get(self) -> Index {
         let i = unsafe { rb_enc_get_index(self.as_rb_value()) };
@@ -1178,16 +1249,17 @@ pub trait EncodingCapable: ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{
-    ///     encoding::{self, EncodingCapable},
-    ///     RString,
-    /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{encoding::EncodingCapable, Error, Ruby};
     ///
-    /// let s = RString::new("example");
-    /// assert!(s.enc_get() == encoding::Index::utf8());
-    /// s.enc_set(encoding::Index::usascii()).unwrap();
-    /// assert!(s.enc_get() == encoding::Index::usascii());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.str_new("example");
+    ///     assert!(s.enc_get() == ruby.utf8_encindex());
+    ///     s.enc_set(ruby.usascii_encindex())?;
+    ///     assert!(s.enc_get() == ruby.usascii_encindex());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn enc_set<T>(self, enc: T) -> Result<(), Error>
     where
@@ -1213,16 +1285,17 @@ pub trait EncodingCapable: ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{
-    ///     encoding::{self, EncodingCapable},
-    ///     RString,
-    /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{encoding::EncodingCapable, Error, Ruby};
     ///
-    /// let s = RString::new("example");
-    /// assert!(s.enc_get() == encoding::Index::utf8());
-    /// s.enc_associate(encoding::Index::usascii()).unwrap();
-    /// assert!(s.enc_get() == encoding::Index::usascii());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.str_new("example");
+    ///     assert!(s.enc_get() == ruby.utf8_encindex());
+    ///     s.enc_associate(ruby.usascii_encindex())?;
+    ///     assert!(s.enc_get() == ruby.usascii_encindex());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn enc_associate<T>(self, enc: T) -> Result<(), Error>
     where
@@ -1244,16 +1317,20 @@ pub trait EncodingCapable: ReprValue + Copy {
 /// # Examples
 ///
 /// ```
-/// use magnus::{encoding, prelude::*, RString};
-/// # let _cleanup = unsafe { magnus::embed::init() };
+/// use magnus::{encoding, prelude::*, Error, Ruby};
 ///
-/// let a = RString::new("a");
-/// let b = RString::new("b");
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let a = ruby.str_new("a");
+///     let b = ruby.str_new("b");
 ///
-/// assert!(a.enc_get() == encoding::Index::utf8());
-/// b.enc_set(encoding::Index::usascii()).unwrap();
+///     assert!(a.enc_get() == ruby.utf8_encindex());
+///     b.enc_set(ruby.usascii_encindex())?;
 ///
-/// assert_eq!(encoding::compatible(a, b).unwrap().name(), "UTF-8");
+///     assert_eq!(encoding::compatible(a, b).unwrap().name(), "UTF-8");
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub fn compatible<T, U>(v1: T, v2: U) -> Option<RbEncoding>
 where
@@ -1272,16 +1349,20 @@ where
 /// # Examples
 ///
 /// ```
-/// use magnus::{encoding, prelude::*, RString};
-/// # let _cleanup = unsafe { magnus::embed::init() };
+/// use magnus::{encoding, prelude::*, Error, Ruby};
 ///
-/// let a = RString::new("a");
-/// let b = RString::new("b");
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let a = ruby.str_new("a");
+///     let b = ruby.str_new("b");
 ///
-/// assert!(a.enc_get() == encoding::Index::utf8());
-/// b.enc_set(encoding::Index::usascii()).unwrap();
+///     assert!(a.enc_get() == ruby.utf8_encindex());
+///     b.enc_set(ruby.usascii_encindex())?;
 ///
-/// assert_eq!(encoding::check(a, b).unwrap().name(), "UTF-8");
+///     assert_eq!(encoding::check(a, b)?.name(), "UTF-8");
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub fn check<T, U>(v1: T, v2: U) -> Result<RbEncoding, Error>
 where
@@ -1306,18 +1387,22 @@ where
 /// # Examples
 ///
 /// ```
-/// use magnus::{encoding, prelude::*, RString};
-/// # let _cleanup = unsafe { magnus::embed::init() };
+/// use magnus::{encoding, prelude::*, Error, Ruby};
 ///
-/// let a = RString::new("a");
-/// assert!(a.enc_get() == encoding::Index::utf8());
-/// let b = RString::new("b");
-/// assert!(b.enc_get() == encoding::Index::utf8());
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let a = ruby.str_new("a");
+///     assert!(a.enc_get() == ruby.utf8_encindex());
+///     let b = ruby.str_new("b");
+///     assert!(b.enc_get() == ruby.utf8_encindex());
 ///
-/// a.enc_set(encoding::Index::usascii()).unwrap();
-/// encoding::copy(b, a).unwrap();
+///     a.enc_set(ruby.usascii_encindex())?;
+///     encoding::copy(b, a)?;
 ///
-/// assert!(b.enc_get() == encoding::Index::usascii());
+///     assert!(b.enc_get() == ruby.usascii_encindex());
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub fn copy<T, U>(dst: T, src: U) -> Result<(), Error>
 where
