@@ -5,8 +5,8 @@
 use std::{any::Any, borrow::Cow, ffi::CString, fmt, mem::transmute, os::raw::c_int};
 
 use rb_sys::{
-    rb_bug, rb_ensure, rb_errinfo, rb_exc_raise, rb_iter_break_value, rb_jump_tag, rb_protect,
-    rb_set_errinfo, rb_warning, ruby_special_consts, VALUE,
+    rb_bug, rb_ensure, rb_errinfo, rb_exc_fatal, rb_exc_raise, rb_iter_break_value, rb_jump_tag,
+    rb_protect, rb_set_errinfo, rb_warning, ruby_special_consts, VALUE,
 };
 
 use crate::{
@@ -19,7 +19,7 @@ use crate::{
 };
 
 /// An error returned to indicate an attempt to interact with the Ruby API from
-/// a non-Ruby thread or without aquiring the GVL.
+/// a non-Ruby thread or without acquiring the GVL.
 #[derive(Debug)]
 pub enum RubyUnavailableError {
     /// GVL is not locked.
@@ -53,22 +53,16 @@ impl Ruby {
     /// use magnus::{prelude::*, Error, Ruby};
     ///
     /// fn example(ruby: &Ruby) -> Result<(), Error> {
-    ///     let i: i64 = ruby
-    ///         .range_new(1, 100, false)?
-    ///         .block_call("each", (), |args, _block| {
-    ///             let i = i64::try_convert(*args.get(0).unwrap())?;
-    ///             if i % 3 == 0 && i % 5 == 0 {
-    ///                 // `block_call` takes a function pointer, not a
-    ///                 // closure, so can't capture `ruby`. As we know this
-    ///                 // will always be called from Ruby it's safe to get
-    ///                 // `Ruby` without the checks that we're on a Ruby
-    ///                 // thread.
-    ///                 let ruby = unsafe { Ruby::get_unchecked() };
-    ///                 Err(ruby.iter_break_value(i))
-    ///             } else {
-    ///                 Ok(())
-    ///             }
-    ///         })?;
+    ///     let i: i64 =
+    ///         ruby.range_new(1, 100, false)?
+    ///             .block_call("each", (), |ruby, args, _block| {
+    ///                 let i = i64::try_convert(*args.get(0).unwrap())?;
+    ///                 if i % 3 == 0 && i % 5 == 0 {
+    ///                     Err(ruby.iter_break_value(i))
+    ///                 } else {
+    ///                     Ok(())
+    ///                 }
+    ///             })?;
     ///
     ///     assert_eq!(i, 15);
     ///     Ok(())
@@ -100,7 +94,7 @@ impl Ruby {
 pub type Result<T> = std::result::Result<T, Error>;
 
 /// The possible types of [`Error`].
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum ErrorType {
     /// An interrupt, such as `break` or `throw`.
     Jump(Tag),
@@ -112,7 +106,7 @@ pub enum ErrorType {
 }
 
 /// Wrapper type for Ruby `Exception`s or other interrupts.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Error(ErrorType);
 
 impl Error {
@@ -122,28 +116,32 @@ impl Error {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{define_global_function, eval, exception, function, prelude::*, Error, Exception};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{function, prelude::*, Error, Exception, Ruby};
     ///
-    /// fn bang() -> Result<(), Error> {
-    ///     Err(Error::new(exception::runtime_error(), "BANG"))
+    /// fn bang(ruby: &Ruby) -> Result<(), Error> {
+    ///     Err(Error::new(ruby.exception_runtime_error(), "BANG"))
     /// }
-    /// define_global_function("bang", function!(bang, 0));
     ///
-    /// let error: Exception = eval(
-    ///     "
-    ///       begin
-    ///         bang
-    ///       rescue => e
-    ///         e
-    ///       end
-    ///     ",
-    /// )
-    /// .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     ruby.define_global_function("bang", function!(bang, 0));
     ///
-    /// assert!(error.is_kind_of(exception::runtime_error()));
-    /// let msg: String = error.funcall("message", ()).unwrap();
-    /// assert_eq!(msg, "BANG")
+    ///     let error: Exception = ruby.eval(
+    ///         "
+    ///             begin
+    ///               bang
+    ///             rescue => e
+    ///               e
+    ///             end
+    ///             ",
+    ///     )?;
+    ///
+    ///     assert!(error.is_kind_of(ruby.exception_runtime_error()));
+    ///     let msg: String = error.funcall("message", ())?;
+    ///     assert_eq!(msg, "BANG");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn new<T>(class: ExceptionClass, msg: T) -> Self
     where
@@ -166,12 +164,13 @@ impl Error {
     /// # Examples
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::{prelude::*, Error};
     /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
     /// let i: i64 = magnus::Range::new(1, 100, false)
     ///     .unwrap()
-    ///     .block_call("each", (), |args, _block| {
+    ///     .block_call("each", (), |_ruby, args, _block| {
     ///         let i = i64::try_convert(*args.get(0).unwrap())?;
     ///         if i % 3 == 0 && i % 5 == 0 {
     ///             Err(Error::iter_break(i))
@@ -184,9 +183,10 @@ impl Error {
     /// assert_eq!(i, 15);
     /// ```
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::iter_break_value` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn iter_break<T>(val: T) -> Self
     where
@@ -201,13 +201,12 @@ impl Error {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{
-    ///     class, eval, exception::ExceptionClass, prelude::*, Error, RModule, TryConvert, Value,
-    /// };
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{exception::ExceptionClass, prelude::*, Error, RModule, Ruby, TryConvert, Value};
     ///
-    /// let err: Error = eval::<Value>(
-    ///     "
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let err: Error = ruby
+    ///         .eval::<Value>(
+    ///             "
     ///       class ExampleError < StandardError
     ///       end
     ///       module Tag
@@ -217,20 +216,24 @@ impl Error {
     ///       end
     ///       raise SpecificError
     ///     ",
-    /// )
-    /// .unwrap_err();
+    ///         )
+    ///         .unwrap_err();
     ///
-    /// fn get<T: TryConvert>(name: &str) -> T {
-    ///     class::object().const_get::<_, T>(name).unwrap()
+    ///     fn get<T: TryConvert>(ruby: &Ruby, name: &str) -> Result<T, Error> {
+    ///         ruby.class_object().const_get::<_, T>(name)
+    ///     }
+    ///     assert!(err.is_kind_of(get::<ExceptionClass>(ruby, "SpecificError")?));
+    ///     assert!(err.is_kind_of(get::<ExceptionClass>(ruby, "ExampleError")?));
+    ///     assert!(err.is_kind_of(get::<ExceptionClass>(ruby, "StandardError")?));
+    ///     assert!(err.is_kind_of(get::<ExceptionClass>(ruby, "Exception")?));
+    ///     assert!(err.is_kind_of(get::<RModule>(ruby, "Tag")?));
+    ///
+    ///     assert!(!err.is_kind_of(get::<ExceptionClass>(ruby, "NoMethodError")?));
+    ///     assert!(!err.is_kind_of(get::<RModule>(ruby, "Math")?));
+    ///
+    ///     Ok(())
     /// }
-    /// assert!(err.is_kind_of(get::<ExceptionClass>("SpecificError")));
-    /// assert!(err.is_kind_of(get::<ExceptionClass>("ExampleError")));
-    /// assert!(err.is_kind_of(get::<ExceptionClass>("StandardError")));
-    /// assert!(err.is_kind_of(get::<ExceptionClass>("Exception")));
-    /// assert!(err.is_kind_of(get::<RModule>("Tag")));
-    ///
-    /// assert!(!err.is_kind_of(get::<ExceptionClass>("NoMethodError")));
-    /// assert!(!err.is_kind_of(get::<RModule>("Math")));
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn is_kind_of<T>(&self, class: T) -> bool
     where
@@ -273,7 +276,7 @@ impl Error {
     ///
     /// This function is provided for rare cases where the `Error` needs to be
     /// stored on the heap and the inner value needs to be
-    /// [marked](`crate::gc::mark`) to avoid being garbage collected.
+    /// [marked](`crate::gc::Marker::mark`) to avoid being garbage collected.
     pub fn value(&self) -> Option<Value> {
         match self.0 {
             ErrorType::Jump(_) => None,
@@ -317,6 +320,19 @@ impl From<Exception> for Error {
     }
 }
 
+/// Conversions into [`Error`].
+pub trait IntoError {
+    /// Convert `self` into [`Error`].
+    fn into_error(self, ruby: &Ruby) -> Error;
+}
+
+impl IntoError for Error {
+    #[inline]
+    fn into_error(self, _: &Ruby) -> Error {
+        self
+    }
+}
+
 /// A wrapper to make a [`Error`] [`Send`] + [`Sync`].
 ///
 /// [`Error`] is not [`Send`] or [`Sync`] as it provides a way to call some of
@@ -334,6 +350,7 @@ impl From<Exception> for Error {
 /// Note that `OpaqueError` contains a Ruby value, so must be kept on the stack
 /// of a Ruby thread to prevent it from being Garbage Collected (or otherwise
 /// protected from premature GC).
+#[derive(Clone)]
 pub struct OpaqueError(ErrorType);
 
 unsafe impl Send for OpaqueError {}
@@ -369,9 +386,16 @@ impl From<Error> for OpaqueError {
     }
 }
 
+impl IntoError for OpaqueError {
+    #[inline]
+    fn into_error(self, _: &Ruby) -> Error {
+        Error(self.0)
+    }
+}
+
 /// The state of a call to Ruby exiting early, interrupting the normal flow
 /// of code.
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
 #[repr(i32)]
 pub enum Tag {
     // None = 0,
@@ -496,13 +520,9 @@ where
 
     let result = unsafe {
         let call_func_ptr = call_func::<F1, T> as unsafe extern "C" fn(VALUE) -> VALUE;
-        #[cfg(ruby_lt_2_7)]
-        let call_func_ptr: unsafe extern "C" fn() -> VALUE = std::mem::transmute(call_func_ptr);
         let mut some_func = Some(func);
         let func_closure = &mut some_func as *mut Option<F1> as VALUE;
         let call_ensure_ptr = call_ensure::<F2> as unsafe extern "C" fn(VALUE) -> VALUE;
-        #[cfg(ruby_lt_2_7)]
-        let call_ensure_ptr: unsafe extern "C" fn() -> VALUE = std::mem::transmute(call_ensure_ptr);
         let mut some_ensure = Some(ensure);
         let ensure_closure = &mut some_ensure as *mut Option<F2> as VALUE;
         rb_ensure(
@@ -519,6 +539,15 @@ where
 pub(crate) fn raise(e: Error) -> ! {
     match e.0 {
         ErrorType::Jump(tag) => tag.resume(),
+        ErrorType::Error(class, _)
+            if class.as_rb_value()
+                == unsafe { Ruby::get_unchecked().exception_fatal().as_rb_value() } =>
+        {
+            unsafe { rb_exc_fatal(e.exception().as_rb_value()) }
+            // friendly reminder: we really never get here, and as such won't
+            // drop any values still in scope, make sure everything has been
+            // consumed/dropped
+        }
         _ => {
             unsafe { rb_exc_raise(e.exception().as_rb_value()) }
             // friendly reminder: we really never get here, and as such won't
@@ -557,9 +586,10 @@ pub fn bug(s: &str) -> ! {
 /// Panics if called from a non-Ruby thread. See [`Ruby::warning`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::warning` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn warning(s: &str) {
     get_ruby!().warning(s)
