@@ -1,4 +1,4 @@
-//! Magnus is a library for writing Ruby extentions in Rust, or running Ruby
+//! Magnus is a library for writing Ruby extensions in Rust, or running Ruby
 //! code from Rust.
 //!
 //! # Overview
@@ -57,7 +57,7 @@
 //! # Examples
 //!
 //! ```
-//! use magnus::{class, define_module, function, method, prelude::*, Error};
+//! use magnus::{function, method, prelude::*, Error, Ruby};
 //!
 //! #[magnus::wrap(class = "Euclid::Point", free_immediately, size)]
 //! struct Point {
@@ -84,22 +84,22 @@
 //! }
 //!
 //! #[magnus::init]
-//! fn init() -> Result<(), Error> {
-//!     let module = define_module("Euclid")?;
-//!     let class = module.define_class("Point", class::object())?;
+//! fn init(ruby: &Ruby) -> Result<(), Error> {
+//!     let module = ruby.define_module("Euclid")?;
+//!     let class = module.define_class("Point", ruby.class_object())?;
 //!     class.define_singleton_method("new", function!(Point::new, 2))?;
 //!     class.define_method("x", method!(Point::x, 0))?;
 //!     class.define_method("y", method!(Point::y, 0))?;
 //!     module.define_module_function("distance", function!(distance, 2))?;
 //!     Ok(())
 //! }
-//! # let _cleanup = unsafe { magnus::embed::init() };
-//! # init().unwrap();
+//! # Ruby::init(init).unwrap()
 //! ```
 //!
 //! # Crates that work with Magnus
 //!
 //! * [`rb-sys`](https://docs.rs/rb-sys) - low level bindings to Ruby.
+//! * [`serde_magnus`](https://docs.rs/serde_magnus) - Serde integration.
 //!
 //! # C Function Index
 //!
@@ -222,7 +222,7 @@
 // * `rb_add_event_hook`:
 // * `rb_add_event_hook2`:
 //! * `rb_alias`: [`Module::define_alias`].
-// * `rb_alias_variable`:
+//! * `rb_alias_variable`: [`Ruby::alias_variable`].
 // * `RB_ALLOC`:
 // * `RB_ALLOCV`:
 // * `RB_ALLOCV_END`:
@@ -324,7 +324,7 @@
 // * `rb_big_modulo`:
 // * `rb_big_mul`:
 // * `rb_big_new`:
-// * `rb_big_norm`:
+//! * `rb_big_norm`: [`Integer::norm`].
 // * `rb_big_or`:
 // * `rb_big_pack`:
 // * `rb_big_plus`:
@@ -334,7 +334,6 @@
 // * `rb_big_sign`:
 // * `rb_big_unpack`:
 // * `rb_big_xor`:
-//! * `rb_binding_new`: [`Binding::new`]. Unimplemented >= Ruby 3.2.
 //! * `rb_block_call`: See [`Value::block_call`].
 //! * `rb_block_call_kw`: [`Value::block_call`].
 //! * `rb_block_given_p`: [`block::block_given`].
@@ -346,19 +345,19 @@
 //!
 //! ## `rb_c`
 //!
-//! * `rb_call_super`: See [`call_super`].
-//! * `rb_call_super_kw`: [`call_super`].
+//! * `rb_call_super`: See [`Ruby::call_super`].
+//! * `rb_call_super_kw`: [`Ruby::call_super`].
 // * `rb_catch`:
 // * `rb_catch_obj`:
 // * `rb_category_compile_warn`:
 // * `rb_category_warn`:
 // * `rb_category_warning`:
 // * `rb_char_to_option_kcode`:
-//! * `rb_check_arity`: [`scan_args::check_arity`].
+//! * `rb_check_arity`: [`Ruby::check_arity`].
 //! * `rb_check_array_type`:  See [`TryConvert`] and [`Value::try_convert`].
 // * `rb_check_convert_type`:
 // * `rb_check_copyable`:
-// * `rb_check_frozen`:
+//! * `rb_check_frozen`: See [`ReprValue::check_frozen`].
 // * `rb_check_frozen_inline`:
 //! * `rb_check_funcall`: See [`Value::check_funcall`].
 //! * `rb_check_funcall_kw`:  [`Value::check_funcall`].
@@ -367,11 +366,11 @@
 //! * `rb_check_id_cstr`: [`Id::check`](value::Id::check).
 // * `rb_check_inheritable`:
 // * `rb_check_safe_str`:
-// * `rb_check_string_type`:
+//! * `rb_check_string_type`: See [`TryConvert`] and [`Value::try_convert`].
 //! * `rb_check_symbol`: Similar to [`StaticSymbol::check`].
 //! * `rb_check_symbol_cstr`: [`StaticSymbol::check`].
-// * `rb_check_to_float`:
-// * `rb_check_to_int`:
+//! * `rb_check_to_float`: See [`TryConvert`] and [`Value::try_convert`].
+//! * `rb_check_to_int`: See [`TryConvert`] and [`Value::try_convert`].
 // * `rb_check_to_integer`:
 // * `rb_check_type`:
 //! * `rb_check_typeddata`: See [`TryConvert`] and [`Value::try_convert`].
@@ -381,7 +380,7 @@
 // * `rb_class_get_superclass`:
 // * `rb_class_inherited_p`: [`Module::is_inherited`].
 // * `rb_class_instance_methods`:
-//! * `rb_class_name`: Simmilar to [`Value::classname`].
+//! * `rb_class_name`: Similar to [`Value::classname`].
 //! * `rb_class_new`: [`RClass::new`].
 //! * `rb_class_new_instance`: See [`RClass::new_instance`].
 //! * `rb_class_new_instance_kw`: [`RClass::new_instance`].
@@ -448,7 +447,7 @@
 // * `rb_cstr2inum`:
 // * `rb_cstr_to_dbl`:
 // * `rb_cstr_to_inum`:
-//! * `rb_current_receiver`: [`current_receiver`].
+//! * `rb_current_receiver`: [`Ruby::current_receiver`].
 // * `rb_cvar_defined`:
 // * `rb_cvar_find`:
 // * `rb_cvar_get`:
@@ -458,6 +457,7 @@
 //!
 //! ## `rb_d`
 //!
+//! * `rb_data_define`: [`Ruby::define_data`].
 //! * `rb_data_object_make`: See [`wrap`] and [`TypedData`].
 //! * `rb_data_object_wrap`: See [`wrap`] and [`TypedData`].
 //! * `rb_data_object_zalloc`: See [`wrap`] and [`TypedData`].
@@ -481,8 +481,8 @@
 //! * `rb_define_alias`: [`Module::define_alias`].
 // * `rb_define_alloc_func`:
 //! * `rb_define_attr`: See [`Module::define_attr`].
-//! * `rb_define_class`: [`define_class`].
-//! * `rb_define_class_id`: Simmilar to [`define_class`].
+//! * `rb_define_class`: [`Ruby::define_class`].
+//! * `rb_define_class_id`: Similar to [`Ruby::define_class`].
 //! * `rb_define_class_id_under`: [`Module::define_class`].
 //! * `rb_define_class_under`: See [`Module::define_class`].
 // * `rb_define_class_variable`:
@@ -490,20 +490,20 @@
 // * `rb_define_dummy_encoding`:
 // * `rb_define_finalizer`:
 // * `rb_define_global_const`:
-//! * `rb_define_global_function`: [`define_global_function`].
+//! * `rb_define_global_function`: [`Ruby::define_global_function`].
 // * `rb_define_hooked_variable`:
 //! * `rb_define_method`: See [`Module::define_method`].
 //! * `rb_define_method_id`: [`Module::define_method`].
-//! * `rb_define_module`: [`define_module`].
+//! * `rb_define_module`: [`Ruby::define_module`].
 //! * `rb_define_module_function`: [`RModule::define_module_function`].
-//! * `rb_define_module_id`: See [`define_module`].
+//! * `rb_define_module_id`: See [`Ruby::define_module`].
 //! * `rb_define_module_id_under`: [`Module::define_module`].
 //! * `rb_define_module_under`: See [`Module::define_module`].
 //! * `rb_define_private_method`: [`Module::define_private_method`].
 //! * `rb_define_protected_method`: [`Module::define_protected_method`].
 // * `rb_define_readonly_variable`:
 //! * `rb_define_singleton_method`: [`Object::define_singleton_method`].
-//! * `rb_define_variable`: [`define_variable`].
+//! * `rb_define_variable`: [`Ruby::define_variable`].
 // * `rb_define_virtual_variable`:
 // * `rb_deprecate_constant`:
 // * `rb_detach_process`:
@@ -658,7 +658,7 @@
 //! * `rb_eval_string`: See [`eval()`] or [`eval!`].
 //! * `rb_eval_string_protect`: [`eval()`] or [`eval!`].
 // * `rb_eval_string_wrap`:
-// * `rb_exc_fatal`:
+//! * `rb_exc_fatal`: Return an [`Error`] constructed from [`Ruby::exception_fatal`].
 // * `rb_exc_new`:
 // * `rb_exc_new_cstr`:
 // * `rb_exc_new_str`:
@@ -694,12 +694,13 @@
 // * `rb_fd_term`:
 // * `rb_fd_zero`:
 // * `rb_feature_provided`:
-// * `rb_fiber_alive_p`:
-// * `rb_fiber_current`:
-// * `rb_fiber_new`:
-// * `rb_fiber_raise`:
-// * `rb_fiber_resume`:
-// * `rb_fiber_resume_kw`:
+//! * `rb_fiber_alive_p`: [`Fiber::is_alive`].
+//! * `rb_fiber_current`: [`Ruby::fiber_current`]
+//! * `rb_fiber_new`: See [`Ruby::fiber_new`] & [`Ruby::fiber_new_from_fn`].
+//! * `rb_fiber_new_storage`: [`Ruby::fiber_new`] & [`Ruby::fiber_new_from_fn`].
+//! * `rb_fiber_raise`: [`Fiber::raise`].
+//! * `rb_fiber_resume`: See [`Fiber::resume`].
+//! * `rb_fiber_resume_kw`: [`Fiber::resume`].
 // * `rb_fiber_scheduler_address_resolve`:
 // * `rb_fiber_scheduler_block`:
 // * `rb_fiber_scheduler_close`:
@@ -724,10 +725,10 @@
 // * `rb_fiber_scheduler_process_wait`:
 // * `rb_fiber_scheduler_set`:
 // * `rb_fiber_scheduler_unblock`:
-// * `rb_fiber_transfer`:
-// * `rb_fiber_transfer_kw`:
-// * `rb_fiber_yield`:
-// * `rb_fiber_yield_kw`:
+//! * `rb_fiber_transfer`: See [`Fiber::transfer`].
+//! * `rb_fiber_transfer_kw`: [`Fiber::transfer`].
+//! * `rb_fiber_yield`: See [`Ruby::fiber_yield`].
+//! * `rb_fiber_yield_kw`: [`Ruby::fiber_yield`].
 //! * `rb_filesystem_encindex`: [`encoding::Index::filesystem`].
 //! * `rb_filesystem_encoding`:
 //!   [`RbEncoding::filesystem`](encoding::RbEncoding::filesystem).
@@ -811,29 +812,29 @@
 //!
 //! ## `rb_g`
 //!
-//! * `rb_gc`: [`gc::start`].
-//! * `rb_gc_adjust_memory_usage`: [`gc::adjust_memory_usage`].
+//! * `rb_gc`: [`Ruby::gc_start`].
+//! * `rb_gc_adjust_memory_usage`: [`Ruby::gc_adjust_memory_usage`].
 // * `rb_gc_call_finalizer_at_exit`:
 // * `rb_gc_copy_finalizer`:
-//! * `rb_gc_count`: [`gc::count`].
-//! * `rb_gc_disable`: [`gc::disable`].
-//! * `rb_gc_enable`: [`gc::enable`].
+//! * `rb_gc_count`: [`Ruby::gc_count`].
+//! * `rb_gc_disable`: [`Ruby::gc_disable`].
+//! * `rb_gc_enable`: [`Ruby::gc_enable`].
 // * `RB_GC_GUARD`:
 // * `rb_gc_latest_gc_info`:
-//! * `rb_gc_location`: [`gc::location`].
-//! * `rb_gc_mark`: [`gc::mark`].
-//! * `rb_gc_mark_locations`: [`gc::mark_slice`].
+//! * `rb_gc_location`: [`gc::Compactor::location`].
+//! * `rb_gc_mark`: [`gc::Marker::mark`].
+//! * `rb_gc_mark_locations`: [`gc::Marker::mark_slice`].
 // * `rb_gc_mark_maybe`:
-//! * `rb_gc_mark_movable`: [`gc::mark_movable`].
+//! * `rb_gc_mark_movable`: [`gc::Marker::mark_movable`].
 //! * `rb_gc_register_address`: [`gc::register_address`] or
 //!   [`BoxValue`](value::BoxValue).
 //! * `rb_gc_register_mark_object`: [`gc::register_mark_object`].
-//! * `rb_gc_start`: [`gc::start`].
-//! * `rb_gc_stat`: [`gc::stat`] or [`gc::all_stats`].
+//! * `rb_gc_start`: [`Ruby::gc_start`].
+//! * `rb_gc_stat`: [`Ruby::gc_stat`] or [`Ruby::gc_all_stats`].
 //! * `rb_gc_unregister_address`: [`gc::unregister_address`].
 // * `rb_gc_update_tbl_refs`:
-// * `rb_gc_writebarrier`:
-// * `rb_gc_writebarrier_unprotect`:
+//! * `rb_gc_writebarrier`: [`typed_data::Writebarrier::writebarrier`].
+//! * `rb_gc_writebarrier_unprotect`: [`typed_data::Writebarrier::writebarrier_unprotect`].
 // * `rb_generic_ivar_table`:
 // * `rb_genrand_int32`:
 // * `rb_genrand_real`:
@@ -871,7 +872,7 @@
 // * `rb_hash_ifnone`:
 // * `rb_hash_iter_lev`:
 //! * `rb_hash_lookup`: [`RHash::lookup`].
-//! * `rb_hash_lookup2`: [`RHash::get`].
+//! * `rb_hash_lookup2`: [`RHash::lookup2`].
 //! * `rb_hash_new`: [`RHash::new`].
 //! * `rb_hash_new_capa`: [`RHash::with_capacity`].
 // * `rb_hash_set_ifnone`:
@@ -931,7 +932,7 @@
 // * `rb_io_descriptor`:
 // * `rb_io_eof`:
 // * `rb_io_extract_encoding_option`:
-// * `rb_io_extract_modeenc`:
+//! * `rb_io_extract_modeenc`: [`Ruby::io_extract_modeenc`].
 // * `rb_io_fdopen`:
 // * `rb_io_flush`:
 // * `rb_io_fptr_finalize`:
@@ -1084,13 +1085,13 @@
 // * `rb_mod_sys_fail`:
 // * `rb_mod_sys_fail_str`:
 // * `rb_must_asciicompat`:
-// * `rb_mutex_lock`:
-// * `rb_mutex_locked_p`:
-// * `rb_mutex_new`:
-// * `rb_mutex_sleep`:
-// * `rb_mutex_synchronize`:
-// * `rb_mutex_trylock`:
-// * `rb_mutex_unlock`:
+//! * `rb_mutex_lock`: [`Mutex::lock`].
+//! * `rb_mutex_locked_p`: [`Mutex::is_locked`].
+//! * `rb_mutex_new`: [`Ruby::mutex_new`].
+//! * `rb_mutex_sleep`: [`Mutex::sleep`].
+//! * `rb_mutex_synchronize`: [`Mutex::synchronize`].
+//! * `rb_mutex_trylock`: [`Mutex::trylock`].
+//! * `rb_mutex_unlock`: [`Mutex::unlock`].
 //!
 //! ## `rb_n`
 // * `rb_name_error`:
@@ -1113,10 +1114,6 @@
 // * `rb_native_mutex_unlock`:
 // * `rb_need_block`:
 // * `RB_NEGFIXABLE`:
-// * `RB_NEWOBJ`:
-// * `rb_newobj`:
-// * `RB_NEWOBJ_OF`:
-// * `rb_newobj_of`:
 // * `RB_NIL_P`:
 // * `rb_nogvl`:
 // * `rb_notimplement`:
@@ -1178,7 +1175,7 @@
 // * `rb_obj_instance_eval`:
 // * `rb_obj_instance_exec`:
 // * `rb_obj_instance_variables`:
-// * `rb_obj_is_fiber`:
+//! * `rb_obj_is_fiber`: [`Fiber::from_value`].
 // * `rb_obj_is_instance_of`:
 //! * `rb_obj_is_kind_of`: [`Value::is_kind_of`].
 // * `rb_obj_is_method`:
@@ -1252,7 +1249,7 @@
 // * `rb_ractor_stdin_set`:
 // * `rb_ractor_stdout`:
 // * `rb_ractor_stdout_set`:
-//! * `rb_raise`: Simmilar to returning [`Error`].
+//! * `rb_raise`: Similar to returning [`Error`].
 // * `rb_random_base_init`:
 // * `rb_random_bytes`:
 // * `RB_RANDOM_DATA_INIT_PARENT`:
@@ -1310,8 +1307,8 @@
 // * `rb_remove_event_hook_with_data`:
 // * `rb_remove_method`:
 // * `rb_remove_method_id`:
-//! * `rb_require`: [`require`].
-//! * `rb_require_string`: [`require`].
+//! * `rb_require`: [`Ruby::require`].
+//! * `rb_require_string`: [`Ruby::require`].
 // * `rb_rescue`:
 // * `rb_rescue2`:
 // * `RB_RESERVED_FD_P`:
@@ -1358,7 +1355,7 @@
 // * `rb_struct_alloc_noinit`:
 //! * `rb_struct_aref`: [`RStruct::aref`].
 //! * `rb_struct_aset`: [`RStruct::aset`].
-//! * `rb_struct_define`: [`r_struct::define_struct`].
+//! * `rb_struct_define`: [`Ruby::define_struct`].
 // * `rb_struct_define_under`:
 // * `rb_struct_define_without_accessor`:
 // * `rb_struct_define_without_accessor_under`:
@@ -1412,10 +1409,10 @@
 //! * `rb_str_new`: [`RString::from_slice`].
 // * `rb_str_new_cstr`:
 //! * `rb_str_new_frozen`: [`RString::new_frozen`].
-//! * `rb_str_new_lit`: Simmilar to [`r_string!`].
-//! * `rb_str_new_literal`: Simmilar to [`r_string!`].
+//! * `rb_str_new_lit`: Similar to [`r_string!`].
+//! * `rb_str_new_literal`: Similar to [`r_string!`].
 //! * `rb_str_new_shared`: [`RString::new_shared`].
-// * `rb_str_new_static`: Simmilar to [`r_string!`].
+// * `rb_str_new_static`: Similar to [`r_string!`].
 // * `rb_str_new_with_class`:
 //! * `rb_str_offset`: [`RString::offset`].
 //! * `rb_str_plus`: [`RString::plus`].
@@ -1506,47 +1503,47 @@
 //! * `RB_TEST`: [`Value::to_bool`] / [`TryConvert`] / [`Value::try_convert`].
 // * `rb_thread_add_event_hook`:
 // * `rb_thread_add_event_hook2`:
-// * `rb_thread_alone`:
+//! * `rb_thread_alone`: [`Ruby::thread_alone`].
 // * `rb_thread_atfork`:
 // * `rb_thread_atfork_before_exec`:
 // * `rb_thread_call_without_gvl`:
 // * `rb_thread_call_without_gvl2`:
 // * `rb_thread_call_with_gvl`:
-// * `rb_thread_check_ints`:
-// * `rb_thread_create`:
-// * `rb_thread_current`:
-// * `rb_thread_fd_close`:
+//! * `rb_thread_check_ints`: [`Ruby::thread_check_ints`].
+//! * `rb_thread_create`: [`Ruby::thread_create`] & [`Ruby::thread_create_from_fn`].
+//! * `rb_thread_current`: [`Ruby::thread_current`].
+//! * `rb_thread_fd_close`: [`Ruby::thread_fd_close`].
 // * `rb_thread_fd_select`:
-// * `rb_thread_fd_writable`:
-// * `rb_thread_interrupted`:
-// * `rb_thread_kill`:
-// * `rb_thread_local_aref`:
-// * `rb_thread_local_aset`:
-// * `rb_thread_main`:
+//! * `rb_thread_fd_writable`: [`Ruby::thread_fd_writable`].
+//! * `rb_thread_interrupted`: [`Thread::interrupted`].
+//! * `rb_thread_kill`: [`Thread::kill`].
+//! * `rb_thread_local_aref`: [`Thread::local_aref`].
+//! * `rb_thread_local_aset`: [`Thread::local_aset`].
+//! * `rb_thread_main`: [`Ruby::thread_main`].
 // * `rb_thread_remove_event_hook`:
 // * `rb_thread_remove_event_hook_with_data`:
-// * `rb_thread_run`:
-// * `rb_thread_schedule`:
-// * `rb_thread_sleep`:
-// * `rb_thread_sleep_deadly`:
-// * `rb_thread_sleep_forever`:
-// * `rb_thread_stop`:
-// * `rb_thread_wait_fd`:
-// * `rb_thread_wait_for`:
-// * `rb_thread_wakeup`:
-// * `rb_thread_wakeup_alive`:
+//! * `rb_thread_run`: [`Thread::run`].
+//! * `rb_thread_schedule`: [`Ruby::thread_schedule`].
+//! * `rb_thread_sleep`: See [`Ruby::thread_sleep`].
+//! * `rb_thread_sleep_deadly`: [`Ruby::thread_sleep_deadly`].
+//! * `rb_thread_sleep_forever`: [`Ruby::thread_sleep_forever`].
+//! * `rb_thread_stop`: [`Ruby::thread_stop`].
+//! * `rb_thread_wait_fd`: [`Ruby::thread_wait_fd`].
+//! * `rb_thread_wait_for`: [`Ruby::thread_sleep`].
+//! * `rb_thread_wakeup`: [`Thread::wakeup`].
+//! * `rb_thread_wakeup_alive`: [`Thread::wakeup_alive`].
 // * `rb_throw`:
 // * `rb_throw_obj`:
 // * `rb_timespec_now`:
 // * `rb_time_interval`:
-// * `rb_time_nano_new`:
-// * `rb_time_new`:
+//! * `rb_time_nano_new`: [`Ruby::time_nano_new`].
+//! * `rb_time_new`: [`Ruby::time_new`].
 // * `rb_time_num_new`:
-// * `rb_time_timespec`:
+//! * `rb_time_timespec`: See [`Time::timespec`].
 // * `rb_time_timespec_interval`:
-// * `rb_time_timespec_new`:
+//! * `rb_time_timespec_new`: [`Ruby::time_timespec_new`].
 // * `rb_time_timeval`:
-// * `rb_time_utc_offset`:
+//! * `rb_time_utc_offset`: [`Time::utc_offset`].
 // * `rb_tolower`:
 // * `rb_toupper`:
 //! * `rb_to_encoding`: [`TryConvert`] or [`Value::try_convert`].
@@ -1609,8 +1606,8 @@
 //! * `rb_utf8_encoding`: [`RbEncoding::utf8`](encoding::RbEncoding::utf8).
 //! * `rb_utf8_str_new`: [`RString::new`].
 //! * `rb_utf8_str_new_cstr`: See [`RString::new`].
-//! * `rb_utf8_str_new_lit`: Simmilar to [`r_string!`].
-//! * `rb_utf8_str_new_literal`: Simmilar to [`r_string!`].
+//! * `rb_utf8_str_new_lit`: Similar to [`r_string!`].
+//! * `rb_utf8_str_new_literal`: Similar to [`r_string!`].
 //! * `rb_utf8_str_new_static`: [`r_string!`].
 // * `rb_uv_to_utf8`:
 //!
@@ -1619,21 +1616,21 @@
 // * `rb_vsprintf`:
 // * `rb_w32_fd_copy`:
 // * `rb_w32_fd_dup`:
-// * `rb_waitpid`:
+//! * `rb_waitpid`: [`Ruby::waitpid`].
 // * `rb_warn`:
-//! * `rb_warning`: [`error::warning`].
+//! * `rb_warning`: [`Ruby::warning`].
 // * `rb_write_error`:
 // * `rb_write_error2`:
-//! * `rb_yield`: [`block::yield_value`] / return [`block::Yield`].
+//! * `rb_yield`: [`Ruby::yield_value`] / return [`block::Yield`].
 // * `rb_yield_block`:
-//! * `rb_yield_splat`: [`block::yield_splat`] / return [`block::YieldSplat`].
+//! * `rb_yield_splat`: [`Ruby::yield_splat`] / return [`block::YieldSplat`].
 // * `rb_yield_splat_kw`:
 //! * `rb_yield_values`:
-//!   See [`block::yield_values`] / return [`block::YieldValues`].
+//!   See [`Ruby::yield_values`] / return [`block::YieldValues`].
 //! * `rb_yield_values2`:
-//!   See [`block::yield_values`] / return [`block::YieldValues`].
+//!   See [`Ruby::yield_values`] / return [`block::YieldValues`].
 //! * `rb_yield_values_kw`:
-//!   [`block::yield_values`] / return [`block::YieldValues`].
+//!   [`Ruby::yield_values`] / return [`block::YieldValues`].
 // * `RB_ZALLOC`:
 // * `RB_ZALLOC_N`:
 //!
@@ -1739,11 +1736,11 @@
 // * `ruby_scan_digits`:
 // * `ruby_scan_hex`:
 // * `ruby_scan_oct`:
-//! * `ruby_script`: Similar to [`embed::ruby_script`].
+//! * `ruby_script`: Similar to [`Ruby::script`].
 // * `ruby_setenv`:
 //! * `ruby_setup`: [`embed::setup`].
 // * `ruby_set_argv`:
-//! * `ruby_set_script_name`: [`embed::ruby_script`].
+//! * `ruby_set_script_name`: [`Ruby::script`].
 // * `ruby_show_copyright`:
 // * `ruby_show_version`:
 // * `ruby_signal_name`:
@@ -1791,7 +1788,6 @@
 mod macros;
 
 mod api;
-mod binding;
 pub mod block;
 pub mod class;
 #[cfg(feature = "embed")]
@@ -1801,14 +1797,22 @@ pub mod encoding;
 mod enumerator;
 pub mod error;
 pub mod exception;
+#[cfg(any(ruby_gte_3_1, docsrs))]
+#[cfg_attr(docsrs, doc(cfg(ruby_gte_3_1)))]
+pub mod fiber;
 mod float;
 pub mod gc;
 mod integer;
 mod into_value;
+#[cfg(feature = "io")]
+#[cfg_attr(docsrs, doc(cfg(feature = "io")))]
+pub mod io;
 pub mod method;
 pub mod module;
+mod mutex;
 pub mod numeric;
 mod object;
+pub mod process;
 /// Traits that commonly should be in scope.
 pub mod prelude {
     pub use crate::{
@@ -1817,7 +1821,7 @@ pub mod prelude {
         value::ReprValue as _,
     };
 }
-mod r_array;
+pub mod r_array;
 mod r_bignum;
 mod r_complex;
 mod r_file;
@@ -1836,23 +1840,24 @@ mod range;
 pub mod rb_sys;
 pub mod scan_args;
 pub mod symbol;
+mod thread;
+pub mod time;
 pub mod try_convert;
 pub mod typed_data;
 pub mod value;
 
 use std::{ffi::CString, mem::transmute, os::raw::c_int};
 
-#[cfg(ruby_lt_2_7)]
-use ::rb_sys::rb_require;
-#[cfg(ruby_gte_2_7)]
-use ::rb_sys::rb_require_string;
 use ::rb_sys::{
-    rb_backref_get, rb_call_super_kw, rb_current_receiver, rb_define_class, rb_define_global_const,
-    rb_define_global_function, rb_define_module, rb_define_variable, rb_errinfo,
-    rb_eval_string_protect, rb_set_errinfo, VALUE,
+    rb_alias_variable, rb_backref_get, rb_call_super_kw, rb_current_receiver, rb_define_class,
+    rb_define_global_const, rb_define_global_function, rb_define_module, rb_define_variable,
+    rb_errinfo, rb_eval_string_protect, rb_require_string, rb_set_errinfo, VALUE,
 };
 pub use magnus_macros::{init, wrap, DataTypeFunctions, TypedData};
 
+#[cfg(any(ruby_gte_3_1, docsrs))]
+#[cfg_attr(docsrs, doc(cfg(ruby_gte_3_1)))]
+pub use crate::fiber::Fiber;
 #[cfg(ruby_use_flonum)]
 pub use crate::value::Flonum;
 pub use crate::{
@@ -1865,6 +1870,7 @@ pub use crate::{
     integer::Integer,
     into_value::{ArgList, IntoValue, IntoValueFromNative, KwArgs, RArrayArgList},
     module::{Attr, Module, RModule},
+    mutex::Mutex,
     numeric::Numeric,
     object::Object,
     r_array::RArray,
@@ -1882,38 +1888,103 @@ pub use crate::{
     r_typed_data::RTypedData,
     range::Range,
     symbol::Symbol,
+    thread::Thread,
+    time::Time,
     try_convert::TryConvert,
     typed_data::{DataType, DataTypeFunctions, TypedData},
     value::{Fixnum, StaticSymbol, Value},
 };
-#[allow(deprecated)]
-pub use crate::{
-    binding::Binding,
-    value::{QFALSE, QNIL, QTRUE},
-};
 use crate::{
     error::protect,
     method::Method,
     r_string::IntoRString,
-    value::{private::ReprValue as _, ReprValue},
+    value::{private::ReprValue as _, IntoId, ReprValue},
 };
 
-#[doc(hidden)]
-#[deprecated(
-    since = "0.6.0",
-    note = "please use `value::Lazy` or `DataType::builder` `const` constructors to set a `static` instead"
-)]
+/// Evaluate a literal string of Ruby code with the given local variables.
+///
+/// Any type that implements [`IntoValue`] can be passed to Ruby.
+///
+/// See also the [`eval`](fn@crate::eval) function.
+///
+/// # Panics
+///
+/// Panics if called from a non-Ruby thread.
+///
+/// # Examples
+///
+/// ```
+/// # let _cleanup = unsafe { magnus::embed::init() };
+/// let result: i64 = magnus::eval!("a + b", a = 1, b = 2).unwrap();
+/// assert_eq!(result, 3)
+/// ```
+/// ```
+/// # let _cleanup = unsafe { magnus::embed::init() };
+/// let a = 1;
+/// let b = 2;
+/// let result: i64 = magnus::eval!("a + b", a, b).unwrap();
+/// assert_eq!(result, 3);
+/// ```
 #[macro_export]
-macro_rules! memoize {
-    ($type:ty: $val:expr) => {{
-        static INIT: std::sync::Once = std::sync::Once::new();
-        static mut VALUE: Option<$type> = None;
-        unsafe {
-            INIT.call_once(|| {
-                VALUE = Some($val);
-            });
-            VALUE.as_ref().unwrap()
-        }
+macro_rules! eval {
+    ($str:literal) => {{
+        $crate::eval!($crate::Ruby::get().unwrap(), $str)
+    }};
+    ($str:literal, $($bindings:tt)*) => {{
+        $crate::eval!($crate::Ruby::get().unwrap(), $str, $($bindings)*)
+    }};
+    ($ruby:expr, $str:literal) => {{
+        use $crate::{r_string::IntoRString, value::ReprValue};
+        $ruby
+            .eval::<$crate::Value>("binding")
+            .unwrap()
+            .funcall("eval", ($str.into_r_string_with(&$ruby),))
+    }};
+    ($ruby:expr, $str:literal, $($bindings:tt)*) => {{
+        use $crate::{r_string::IntoRString, value::ReprValue};
+        let binding = $ruby.eval::<$crate::Value>("binding").unwrap();
+        $crate::bind!(binding, $($bindings)*);
+        binding.funcall("eval", ($str.into_r_string_with(&$ruby),))
+    }};
+}
+
+#[doc(hidden)]
+#[macro_export]
+macro_rules! bind {
+    ($binding:ident,) => {};
+    ($binding:ident, $k:ident = $v:expr) => {{
+        use $crate::symbol::IntoSymbol;
+        let _: $crate::Value = $binding.funcall(
+            "local_variable_set",
+            (stringify!($k).into_symbol_with(&$crate::Ruby::get_with($binding)), $v),
+        )
+        .unwrap();
+    }};
+    ($binding:ident, $k:ident) => {{
+        use $crate::symbol::IntoSymbol;
+        let _: $crate::Value = $binding.funcall(
+            "local_variable_set",
+            (stringify!($k).into_symbol_with(&$crate::Ruby::get_with($binding)), $k),
+        )
+        .unwrap();
+    }};
+    ($binding:ident, $k:ident = $v:expr, $($rest:tt)*) => {{
+        use $crate::symbol::IntoSymbol;
+        let _: $crate::Value = $binding.funcall(
+            "local_variable_set",
+            (stringify!($k).into_symbol_with(&$crate::Ruby::get_with($binding)), $v),
+        )
+        .unwrap();
+        $crate::bind!($binding, $($rest)*);
+    }};
+    ($binding:ident, $k:ident, $($rest:tt)*) => {{
+        use $crate::symbol::IntoSymbol;
+        let _: $crate::Value = $binding.funcall(
+            "local_variable_set",
+            (stringify!($k).into_symbol_with(&$crate::Ruby::get_with($binding)), $k),
+        )
+        .unwrap();
+        $crate::bind!($binding, $($rest)*);
     }};
 }
 
@@ -2109,6 +2180,40 @@ impl Ruby {
         Ok(ptr)
     }
 
+    /// Alias the global variable `src` as `dst`.
+    ///
+    /// Unlike [`define_variable`](Ruby::define_variable), the preceding `$`
+    /// of the global variable's name is required, otherwise the alias will not
+    /// be accessible from Ruby.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use magnus::{rb_assert, Error, Ruby};
+    ///
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     ruby.define_variable("example", 42)?;
+    ///     ruby.alias_variable("$answer", "$example")?;
+    ///     rb_assert!(ruby, "$answer == 42");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
+    /// ```
+    pub fn alias_variable<T, U>(&self, dst: T, src: U) -> Result<(), Error>
+    where
+        T: IntoId,
+        U: IntoId,
+    {
+        let dst = dst.into_id_with(self);
+        let src = src.into_id_with(self);
+        protect(|| {
+            unsafe { rb_alias_variable(dst.as_rb_id(), src.as_rb_id()) };
+            self.qnil()
+        })?;
+        Ok(())
+    }
+
     /// Define a global constant.
     ///
     /// # Examples
@@ -2298,7 +2403,6 @@ impl Ruby {
     /// }
     /// # Ruby::init(example).unwrap()
     /// ```
-    #[cfg(ruby_gte_2_7)]
     pub fn require<T>(&self, feature: T) -> Result<bool, Error>
     where
         T: IntoRString,
@@ -2308,33 +2412,12 @@ impl Ruby {
             .and_then(TryConvert::try_convert)
     }
 
-    /// Finds and loads the given feature if not already loaded.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use magnus::{rb_assert, Error, Ruby};
-    ///
-    /// fn example(ruby: &Ruby) -> Result<(), Error> {
-    ///     assert!(ruby.require("net/http")?);
-    ///
-    ///     Ok(())
-    /// }
-    /// # Ruby::init(example).unwrap()
-    /// ```
-    #[cfg(ruby_lt_2_7)]
-    pub fn require(&self, feature: &str) -> Result<bool, Error> {
-        let feature = CString::new(feature).unwrap();
-        protect(|| unsafe { Value::new(rb_require(feature.as_ptr())) })
-            .and_then(TryConvert::try_convert)
-    }
-
     /// Evaluate a string of Ruby code, converting the result to a `T`.
     ///
     /// Ruby will use the 'ASCII-8BIT' (aka binary) encoding for any Ruby
     /// string literals in the passed string of Ruby code. See the
-    /// [`eval`](macro@crate::eval) macro or [`Binding::eval`] for an
-    /// alternative that supports utf-8.
+    /// [`eval`](macro@crate::eval) macro for an alternative that supports
+    /// utf-8.
     ///
     /// Errors if `s` contains a null byte, the conversion fails, or on an
     /// uncaught Ruby exception.
@@ -2387,6 +2470,7 @@ impl Ruby {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{class, define_class, rb_assert};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -2394,9 +2478,10 @@ impl Ruby {
 /// rb_assert!("Example.is_a?(Class)");
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::define_class` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn define_class(name: &str, superclass: RClass) -> Result<RClass, Error> {
     get_ruby!().define_class(name, superclass)
@@ -2412,6 +2497,7 @@ pub fn define_class(name: &str, superclass: RClass) -> Result<RClass, Error> {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{define_module, rb_assert};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -2420,9 +2506,10 @@ pub fn define_class(name: &str, superclass: RClass) -> Result<RClass, Error> {
 /// rb_assert!("!Example.is_a?(Class)");
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::define_module` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn define_module(name: &str) -> Result<RModule, Error> {
     get_ruby!().define_module(name)
@@ -2438,6 +2525,7 @@ pub fn define_module(name: &str) -> Result<RModule, Error> {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{define_error, exception, rb_assert};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -2446,9 +2534,10 @@ pub fn define_module(name: &str) -> Result<RModule, Error> {
 /// rb_assert!("ExampleError < Exception");
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::define_error` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn define_error(name: &str, superclass: ExceptionClass) -> Result<ExceptionClass, Error> {
     get_ruby!().define_error(name, superclass)
@@ -2464,6 +2553,7 @@ pub fn define_error(name: &str, superclass: ExceptionClass) -> Result<ExceptionC
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{define_variable, prelude::*, rb_assert, RString};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -2477,9 +2567,10 @@ pub fn define_error(name: &str, superclass: ExceptionClass) -> Result<ExceptionC
 /// rb_assert!(r#"$example == "answer""#);
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::define_variable` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn define_variable<T>(name: &str, initial: T) -> Result<*mut Value, Error>
 where
@@ -2498,6 +2589,7 @@ where
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{define_global_const, rb_assert};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -2505,9 +2597,10 @@ where
 /// rb_assert!("EXAMPLE == 42");
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::define_global_const` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn define_global_const<T>(name: &str, value: T) -> Result<(), Error>
 where
@@ -2526,6 +2619,7 @@ where
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{define_global_function, function, rb_assert};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -2537,9 +2631,10 @@ where
 /// rb_assert!(r#"greet("world") == "Hello, world!""#);
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::define_global_function` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn define_global_function<M>(name: &str, func: M)
 where
@@ -2558,6 +2653,7 @@ where
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{backref_get, RRegexp};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -2576,9 +2672,10 @@ where
 /// );
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::backref_get` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn backref_get() -> Option<RMatch> {
     get_ruby!().backref_get()
@@ -2596,6 +2693,7 @@ pub fn backref_get() -> Option<RMatch> {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{
 ///     current_receiver, define_global_function, method, prelude::*, rb_assert, Error, Value,
 /// };
@@ -2609,9 +2707,10 @@ pub fn backref_get() -> Option<RMatch> {
 /// rb_assert!("example");
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::current_receiver` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn current_receiver<T>() -> Result<T, Error>
 where
@@ -2634,6 +2733,7 @@ where
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{call_super, define_class, eval, function, prelude::*, rb_assert, Error};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -2659,9 +2759,10 @@ where
 /// rb_assert!(r#"B.new.example == "Hello from A, and hello from B""#)
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::call_super` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn call_super<A, T>(args: A) -> Result<T, Error>
 where
@@ -2681,16 +2782,17 @@ where
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// # let _cleanup = unsafe { magnus::embed::init() };
 /// use magnus::require;
 ///
 /// assert!(require("net/http").unwrap());
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::require` instead")
 )]
-#[cfg(ruby_gte_2_7)]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn require<T>(feature: T) -> Result<bool, Error>
 where
@@ -2699,31 +2801,6 @@ where
     get_ruby!().require(feature)
 }
 
-/// Finds and loads the given feature if not already loaded.
-///
-/// # Panics
-///
-/// Panics if called from a non-Ruby thread. See [`Ruby::require`] for the
-/// non-panicking version.
-///
-/// # Examples
-///
-/// ```
-/// # let _cleanup = unsafe { magnus::embed::init() };
-/// use magnus::require;
-///
-/// assert!(require("net/http").unwrap());
-/// ```
-#[cfg_attr(
-    not(feature = "friendly-api"),
-    deprecated(note = "please use `Ruby::require` instead")
-)]
-#[cfg(ruby_lt_2_7)]
-#[inline]
-pub fn require(feature: &str) -> Result<bool, Error> {
-    get_ruby!().require(feature)
-}
-
 /// Evaluate a string of Ruby code, converting the result to a `T`.
 ///
 /// Ruby will use the 'ASCII-8BIT' (aka binary) encoding for any Ruby string
