@@ -60,7 +60,7 @@ impl Ruby {
 ///
 /// See the [`Module`] trait for defining instance methods and nested
 /// classes/modules.
-/// See the [`Object`] trait for defining singlton methods (aka class methods).
+/// See the [`Object`] trait for defining singleton methods (aka class methods).
 ///
 /// See the [`ReprValue`] trait for additional methods available on this type.
 /// See [`Ruby`](Ruby#rmodule) for methods to create an `RModule`.
@@ -104,6 +104,7 @@ impl RModule {
     /// # Examples
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::{class, prelude::*, RModule};
     /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
@@ -111,9 +112,10 @@ impl RModule {
     /// assert!(module.is_kind_of(class::module()));
     /// ```
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::module_new` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn new() -> Self {
         get_ruby!().module_new()
@@ -126,26 +128,29 @@ impl RModule {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{define_module, function, r_string, rb_assert, RString};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{function, r_string, rb_assert, Error, RString, Ruby};
     ///
-    /// fn greet() -> RString {
-    ///     r_string!("Hello, world!")
+    /// fn greet(ruby: &Ruby) -> RString {
+    ///     r_string!(ruby, "Hello, world!")
     /// }
     ///
-    /// let module = define_module("Greeting").unwrap();
-    /// module
-    ///     .define_module_function("greet", function!(greet, 0))
-    ///     .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let module = ruby.define_module("Greeting")?;
+    ///     module.define_module_function("greet", function!(greet, 0))?;
     ///
-    /// rb_assert!(r#"Greeting.greet == "Hello, world!""#);
+    ///     rb_assert!(ruby, r#"Greeting.greet == "Hello, world!""#);
     ///
-    /// rb_assert!(
-    ///     r#"
-    ///     include Greeting
-    ///     greet == "Hello, world!"
-    /// "#,
-    /// );
+    ///     rb_assert!(
+    ///         ruby,
+    ///         r#"
+    ///             include Greeting
+    ///             greet == "Hello, world!"
+    ///         "#,
+    ///     );
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn define_module_function<M>(self, name: &str, func: M) -> Result<(), Error>
     where
@@ -214,12 +219,16 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, define_module, prelude::*, rb_assert};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, rb_assert, Error, Ruby};
     ///
-    /// let outer = define_module("Outer").unwrap();
-    /// outer.define_class("Inner", class::object()).unwrap();
-    /// rb_assert!("Outer::Inner.is_a?(Class)");
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let outer = ruby.define_module("Outer")?;
+    ///     outer.define_class("Inner", ruby.class_object())?;
+    ///     rb_assert!(ruby, "Outer::Inner.is_a?(Class)");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_class<T>(self, name: T, superclass: RClass) -> Result<RClass, Error>
     where
@@ -243,13 +252,17 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{define_module, prelude::*, rb_assert};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, rb_assert, Error, Ruby};
     ///
-    /// let outer = define_module("Outer").unwrap();
-    /// outer.define_module("Inner").unwrap();
-    /// rb_assert!("Outer::Inner.is_a?(Module)");
-    /// rb_assert!("!Outer::Inner.is_a?(Class)");
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let outer = ruby.define_module("Outer")?;
+    ///     outer.define_module("Inner")?;
+    ///     rb_assert!(ruby, "Outer::Inner.is_a?(Module)");
+    ///     rb_assert!(ruby, "!Outer::Inner.is_a?(Class)");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_module<T>(self, name: T) -> Result<RModule, Error>
     where
@@ -269,15 +282,17 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{define_module, exception, prelude::*, rb_assert};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, rb_assert, Error, Ruby};
     ///
-    /// let outer = define_module("Outer").unwrap();
-    /// outer
-    ///     .define_error("InnerError", exception::standard_error())
-    ///     .unwrap();
-    /// rb_assert!("Outer::InnerError.is_a?(Class)");
-    /// rb_assert!("Outer::InnerError < Exception");
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let outer = ruby.define_module("Outer")?;
+    ///     outer.define_error("InnerError", ruby.exception_standard_error())?;
+    ///     rb_assert!(ruby, "Outer::InnerError.is_a?(Class)");
+    ///     rb_assert!(ruby, "Outer::InnerError < Exception");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_error<T>(self, name: T, superclass: ExceptionClass) -> Result<ExceptionClass, Error>
     where
@@ -295,23 +310,25 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, function, prelude::*, rb_assert, RClass, RModule};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{function, prelude::*, rb_assert, Error, RClass, Ruby};
     ///
-    /// fn example() -> i64 {
+    /// fn test() -> i64 {
     ///     42
     /// }
     ///
-    /// let module = RModule::new();
-    /// module
-    ///     .define_method("example", function!(example, 0))
-    ///     .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let module = ruby.module_new();
+    ///     module.define_method("test", function!(test, 0))?;
     ///
-    /// let class = RClass::new(class::object()).unwrap();
-    /// class.include_module(module).unwrap();
+    ///     let class = RClass::new(ruby.class_object())?;
+    ///     class.include_module(module)?;
     ///
-    /// let obj = class.new_instance(()).unwrap();
-    /// rb_assert!("obj.example == 42", obj);
+    ///     let obj = class.new_instance(())?;
+    ///     rb_assert!(ruby, "obj.test == 42", obj);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn include_module(self, module: RModule) -> Result<(), Error> {
         protect(|| {
@@ -329,33 +346,34 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{call_super, eval, function, prelude::*, rb_assert, Error, RClass, RModule};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{function, prelude::*, rb_assert, Error, RClass, Ruby};
     ///
-    /// fn example() -> Result<i64, Error> {
-    ///     Ok(call_super::<_, i64>(())? + 2)
+    /// fn test(ruby: &Ruby) -> Result<i64, Error> {
+    ///     Ok(ruby.call_super::<_, i64>(())? + 2)
     /// }
     ///
-    /// let module = RModule::new();
-    /// module
-    ///     .define_method("example", function!(example, 0))
-    ///     .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let module = ruby.module_new();
+    ///     module.define_method("test", function!(test, 0))?;
     ///
-    /// let class: RClass = eval(
-    ///     r#"
-    ///     class Example
-    ///       def example
-    ///         40
-    ///       end
-    ///     end
-    ///     Example
-    /// "#,
-    /// )
-    /// .unwrap();
-    /// class.prepend_module(module).unwrap();
+    ///     let class: RClass = ruby.eval(
+    ///         r#"
+    ///             class Example
+    ///               def test
+    ///                 40
+    ///               end
+    ///             end
+    ///             Example
+    ///         "#,
+    ///     )?;
+    ///     class.prepend_module(module)?;
     ///
-    /// let obj = class.new_instance(()).unwrap();
-    /// rb_assert!("obj.example == 42", obj);
+    ///     let obj = class.new_instance(())?;
+    ///     rb_assert!("obj.test == 42", obj);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn prepend_module(self, module: RModule) -> Result<(), Error> {
         protect(|| {
@@ -370,12 +388,16 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, rb_assert, Module};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{rb_assert, Error, Module, Ruby};
     ///
-    /// class::array().const_set("EXAMPLE", 42).unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     ruby.class_array().const_set("EXAMPLE", 42)?;
     ///
-    /// rb_assert!("Array::EXAMPLE == 42");
+    ///     rb_assert!(ruby, "Array::EXAMPLE == 42");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn const_set<T, U>(self, name: T, value: U) -> Result<(), Error>
     where
@@ -397,20 +419,23 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, eval, rb_assert, Module, RClass, Value};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{rb_assert, Error, Module, RClass, Ruby, Value};
     ///
-    /// eval::<Value>(
-    ///     "
-    ///     class Example
-    ///       VALUE = 42
-    ///     end
-    /// ",
-    /// )
-    /// .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     ruby.eval::<Value>(
+    ///         "
+    ///           class Example
+    ///             VALUE = 42
+    ///           end
+    ///         ",
+    ///     )?;
     ///
-    /// let class = class::object().const_get::<_, RClass>("Example").unwrap();
-    /// rb_assert!("klass::VALUE == 42", klass = class);
+    ///     let class = ruby.class_object().const_get::<_, RClass>("Example")?;
+    ///     rb_assert!(ruby, "klass::VALUE == 42", klass = class);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn const_get<T, U>(self, name: T) -> Result<U, Error>
     where
@@ -431,13 +456,17 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, prelude::*, Module, RClass};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Module, RClass, Ruby};
     ///
-    /// let a = RClass::new(class::object()).unwrap();
-    /// let b = RClass::new(a).unwrap();
-    /// assert!(b.is_inherited(a));
-    /// assert!(!a.is_inherited(b));
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let a = RClass::new(ruby.class_object())?;
+    ///     let b = RClass::new(a)?;
+    ///     assert!(b.is_inherited(a));
+    ///     assert!(!a.is_inherited(b));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn is_inherited<T>(self, other: T) -> bool
     where
@@ -457,15 +486,20 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, rb_assert, Module};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{rb_assert, Error, Module, Ruby};
     ///
-    /// let ary = class::string().ancestors();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let ary = ruby.class_string().ancestors();
     ///
-    /// rb_assert!(
-    ///     "ary == [String, Comparable, Object, Kernel, BasicObject]",
-    ///     ary,
-    /// );
+    ///     rb_assert!(
+    ///         ruby,
+    ///         "ary == [String, Comparable, Object, Kernel, BasicObject]",
+    ///         ary,
+    ///     );
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn ancestors(self) -> RArray {
         unsafe { RArray::from_rb_value_unchecked(rb_mod_ancestors(self.as_rb_value())) }
@@ -476,20 +510,24 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, rb_assert, method, Module};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{method, rb_assert, Error, Module, Ruby};
     ///
     /// fn escape_unicode(s: String) -> String {
     ///     s.escape_unicode().to_string()
     /// }
     ///
-    /// class::string()
-    ///     .define_method("escape_unicode", method!(escape_unicode, 0))
-    ///     .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     ruby.class_string()
+    ///         .define_method("escape_unicode", method!(escape_unicode, 0))?;
     ///
-    /// rb_assert!(
-    ///     r#""🤖\etest".escape_unicode == "\\u{1f916}\\u{1b}\\u{74}\\u{65}\\u{73}\\u{74}""#,
-    /// );
+    ///     rb_assert!(
+    ///         ruby,
+    ///         r#""🤖\etest".escape_unicode == "\\u{1f916}\\u{1b}\\u{74}\\u{65}\\u{73}\\u{74}""#,
+    ///     );
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_method<T, M>(self, name: T, func: M) -> Result<(), Error>
     where
@@ -518,8 +556,7 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, eval, exception, function, rb_assert, Module, Value};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{eval, function, rb_assert, Error, Module, Ruby, Value};
     ///
     /// fn percent_encode(c: char) -> String {
     ///     if c.is_ascii_alphanumeric() || c == '-' || c == '_' || c == '.' || c == '~' {
@@ -529,26 +566,29 @@ pub trait Module: Object + ReprValue + Copy {
     ///     }
     /// }
     ///
-    /// class::string()
-    ///     .define_private_method("percent_encode_char", function!(percent_encode, 1))
-    ///     .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     ruby.class_string()
+    ///         .define_private_method("percent_encode_char", function!(percent_encode, 1))?;
     ///
-    /// eval::<Value>(
-    ///     r#"
-    ///     class String
-    ///       def percent_encode
-    ///         chars.map {|c| percent_encode_char(c)}.join("")
-    ///       end
-    ///     end
-    /// "#,
-    /// )
-    /// .unwrap();
+    ///     ruby.eval::<Value>(
+    ///         r#"
+    ///             class String
+    ///               def percent_encode
+    ///                 chars.map {|c| percent_encode_char(c)}.join("")
+    ///               end
+    ///             end
+    ///         "#,
+    ///     )?;
     ///
-    /// rb_assert!(r#""foo bar".percent_encode == "foo%20bar""#);
+    ///     rb_assert!(ruby, r#""foo bar".percent_encode == "foo%20bar""#);
     ///
-    /// assert!(eval::<bool>(r#"" ".percent_encode_char(" ")"#)
-    ///     .unwrap_err()
-    ///     .is_kind_of(exception::no_method_error()));
+    ///     assert!(eval::<bool>(r#"" ".percent_encode_char(" ")"#)
+    ///         .unwrap_err()
+    ///         .is_kind_of(ruby.exception_no_method_error()));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_private_method<M>(self, name: &str, func: M) -> Result<(), Error>
     where
@@ -575,8 +615,7 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, eval, exception, method, rb_assert, Module, Value};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{method, rb_assert, Error, Module, Ruby, Value};
     ///
     /// fn escape_unicode(s: String) -> String {
     ///     s.escape_unicode().to_string()
@@ -586,33 +625,35 @@ pub trait Module: Object + ReprValue + Copy {
     ///     c.is_control() || c.is_whitespace()
     /// }
     ///
-    /// class::string()
-    ///     .define_method("escape_unicode", method!(escape_unicode, 0))
-    ///     .unwrap();
-    /// class::string()
-    ///     .define_protected_method("invisible?", method!(is_invisible, 0))
-    ///     .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     ruby.class_string()
+    ///         .define_method("escape_unicode", method!(escape_unicode, 0))?;
+    ///     ruby.class_string()
+    ///         .define_protected_method("invisible?", method!(is_invisible, 0))?;
     ///
-    /// eval::<Value>(
-    ///     r#"
-    ///     class String
-    ///       def escape_invisible
-    ///         chars.map {|c| c.invisible? ? c.escape_unicode : c}.join("")
-    ///       end
-    ///     end
-    /// "#,
-    /// )
-    /// .unwrap();
+    ///     ruby.eval::<Value>(
+    ///         r#"
+    ///             class String
+    ///               def escape_invisible
+    ///                 chars.map {|c| c.invisible? ? c.escape_unicode : c}.join("")
+    ///               end
+    ///             end
+    ///         "#,
+    ///     )?;
     ///
-    /// rb_assert!(
-    ///     r#"
-    ///       "🤖\tfoo bar".escape_invisible == "🤖\\u{9}foo\\u{20}bar"
-    ///     "#,
-    /// );
+    ///     rb_assert!(
+    ///         ruby,
+    ///         r#""🤖\tfoo bar".escape_invisible == "🤖\\u{9}foo\\u{20}bar""#,
+    ///     );
     ///
-    /// assert!(eval::<bool>(r#"" ".invisible?"#)
-    ///     .unwrap_err()
-    ///     .is_kind_of(exception::no_method_error()));
+    ///     assert!(ruby
+    ///         .eval::<bool>(r#"" ".invisible?"#)
+    ///         .unwrap_err()
+    ///         .is_kind_of(ruby.exception_no_method_error()));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_protected_method<M>(self, name: &str, func: M) -> Result<(), Error>
     where
@@ -641,15 +682,19 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, eval, prelude::*, rb_assert, Attr, Module, RClass, Value};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{eval, prelude::*, rb_assert, Attr, Error, Module, RClass, Ruby, Value};
     ///
-    /// let class = RClass::new(class::object()).unwrap();
-    /// class.define_attr("example", Attr::ReadWrite).unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let class = RClass::new(ruby.class_object())?;
+    ///     class.define_attr("example", Attr::ReadWrite)?;
     ///
-    /// let obj = class.new_instance(()).unwrap();
-    /// let _: Value = eval!("obj.example = 42", obj).unwrap();
-    /// rb_assert!("obj.example == 42", obj);
+    ///     let obj = class.new_instance(())?;
+    ///     let _: Value = eval!(ruby, "obj.example = 42", obj)?;
+    ///     rb_assert!(ruby, "obj.example == 42", obj);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_attr<T>(self, name: T, rw: Attr) -> Result<(), Error>
     where
@@ -677,21 +722,23 @@ pub trait Module: Object + ReprValue + Copy {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, function, prelude::*, rb_assert, Module, RClass};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{function, prelude::*, rb_assert, Error, Module, RClass, Ruby};
     ///
-    /// fn example() -> i64 {
+    /// fn test() -> i64 {
     ///     42
     /// }
     ///
-    /// let class = RClass::new(class::object()).unwrap();
-    /// class
-    ///     .define_method("example", function!(example, 0))
-    ///     .unwrap();
-    /// class.define_alias("test", "example").unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let class = RClass::new(ruby.class_object())?;
+    ///     class.define_method("test", function!(test, 0))?;
+    ///     class.define_alias("example", "test")?;
     ///
-    /// let obj = class.new_instance(()).unwrap();
-    /// rb_assert!("obj.test == 42", obj);
+    ///     let obj = class.new_instance(())?;
+    ///     rb_assert!(ruby, "obj.example == 42", obj);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_alias<T, U>(self, dst: T, src: U) -> Result<(), Error>
     where
@@ -948,9 +995,10 @@ impl Ruby {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_comparable`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_comparable` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn comparable() -> RModule {
     get_ruby!().module_comparable()
@@ -963,9 +1011,10 @@ pub fn comparable() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_enumerable`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_enumerable` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn enumerable() -> RModule {
     get_ruby!().module_enumerable()
@@ -978,9 +1027,10 @@ pub fn enumerable() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_errno`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_errno` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn errno() -> RModule {
     get_ruby!().module_errno()
@@ -993,9 +1043,10 @@ pub fn errno() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_file_test`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_file_test` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn file_test() -> RModule {
     get_ruby!().module_file_test()
@@ -1008,9 +1059,10 @@ pub fn file_test() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_gc`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_gc` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn gc() -> RModule {
     get_ruby!().module_gc()
@@ -1023,9 +1075,10 @@ pub fn gc() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_kernel`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_kernel` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn kernel() -> RModule {
     get_ruby!().module_kernel()
@@ -1038,9 +1091,10 @@ pub fn kernel() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_math`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_math` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn math() -> RModule {
     get_ruby!().module_math()
@@ -1053,9 +1107,10 @@ pub fn math() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_process`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_process` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn process() -> RModule {
     get_ruby!().module_process()
@@ -1068,9 +1123,10 @@ pub fn process() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_wait_readable`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_wait_readable` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn wait_readable() -> RModule {
     get_ruby!().module_wait_readable()
@@ -1083,9 +1139,10 @@ pub fn wait_readable() -> RModule {
 /// Panics if called from a non-Ruby thread. See [`Ruby::module_wait_writable`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::module_wait_writable` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn wait_writable() -> RModule {
     get_ruby!().module_wait_writable()
