@@ -18,8 +18,9 @@ use std::{
 #[cfg(ruby_gte_3_0)]
 use rb_sys::rbimpl_typeddata_flags::{self, RUBY_TYPED_FREE_IMMEDIATELY, RUBY_TYPED_WB_PROTECTED};
 use rb_sys::{
-    self, rb_data_type_struct__bindgen_ty_1, rb_data_type_t, rb_obj_reveal,
-    rb_singleton_class_attached, rb_singleton_class_clone, size_t, VALUE,
+    self, rb_data_type_struct__bindgen_ty_1, rb_data_type_t, rb_gc_writebarrier,
+    rb_gc_writebarrier_unprotect, rb_obj_reveal, rb_singleton_class_attached,
+    rb_singleton_class_clone, size_t, RTYPEDDATA_GET_DATA, VALUE,
 };
 
 #[cfg(ruby_lt_3_0)]
@@ -31,7 +32,7 @@ const RUBY_TYPED_WB_PROTECTED: u32 = rb_sys::ruby_fl_type::RUBY_FL_WB_PROTECTED
 use crate::{
     class::RClass,
     error::{bug_from_panic, Error},
-    gc,
+    gc::{self, Mark},
     into_value::IntoValue,
     object::Object,
     r_typed_data::RTypedData,
@@ -134,12 +135,12 @@ where
     /// in your [`mark`](Self::mark) function, you must update them in this
     /// function using [`gc::Compactor::location`].
     ///
-    /// Ruby values would be concidered moveable if marked with the
+    /// Ruby considers values are moveable if marked with the
     /// [`gc::Marker::mark_movable`] function. Other marking functions such as
     /// [`gc::Marker::mark`] will prevent values being moved.
     ///
     /// As it is only safe for this function to receive a shared `&self`
-    /// reference, you must implement interior mutablility to be able to update
+    /// reference, you must implement interior mutability to be able to update
     /// values. This is very hard to do correctly, and it is recommended to
     /// simply avoid using [`gc::Marker::mark_movable`] and `compact`.
     ///
@@ -157,7 +158,7 @@ where
     ///
     /// # Safety
     ///
-    /// `ptr` must be a vaild pointer to a `Box<Self>`, and must not be aliased
+    /// `ptr` must be a valid pointer to a `Box<Self>`, and must not be aliased
     /// This function will free the memory pointed to by `ptr`.
     ///
     /// This function must not panic.
@@ -172,7 +173,7 @@ where
     ///
     /// # Safety
     ///
-    /// `ptr` must be a vaild pointer to a `Self`, and must not be aliased.
+    /// `ptr` must be a valid pointer to a `Self`, and must not be aliased.
     ///
     /// This function must not panic.
     #[doc(hidden)]
@@ -187,7 +188,7 @@ where
     ///
     /// # Safety
     ///
-    /// `ptr` must be a vaild pointer to a `Self`.
+    /// `ptr` must be a valid pointer to a `Self`.
     ///
     /// This function must not panic.
     #[doc(hidden)]
@@ -202,7 +203,7 @@ where
     ///
     /// # Safety
     ///
-    /// `ptr` must be a vaild pointer to a `Self`, and must not be aliased.
+    /// `ptr` must be a valid pointer to a `Self`, and must not be aliased.
     ///
     /// This function must not panic.
     #[doc(hidden)]
@@ -299,7 +300,14 @@ where
 
     /// Enable the 'write barrier protected' flag.
     ///
-    /// You almost certainly don't want to enable this.
+    /// Types that contain Ruby values by default do not participate in
+    /// generational GC (they are scanned every GC). This flag asserts all
+    /// operations that write Ruby values to this type are protected with
+    /// write barriers (see [`Writebarrier::writebarrier`]) so this
+    /// type can participate in generational GC.
+    ///
+    /// The write barrier is hard to get right. Magnus recommends you do not use
+    /// this flag.
     pub const fn wb_protected(mut self) -> Self {
         self.wb_protected = true;
         self
@@ -329,7 +337,6 @@ where
         } else {
             None
         };
-        #[cfg(ruby_gte_2_7)]
         let dcompact = if self.compact {
             Some(T::extern_compact as _)
         } else {
@@ -341,12 +348,8 @@ where
                 dmark,
                 dfree,
                 dsize,
-                #[cfg(ruby_gte_2_7)]
                 dcompact,
-                #[cfg(ruby_gte_2_7)]
                 reserved: [ptr::null_mut(); 1],
-                #[cfg(ruby_lt_2_7)]
-                reserved: [ptr::null_mut(); 2],
             },
             parent: ptr::null(),
             data: ptr::null_mut(),
@@ -533,7 +536,7 @@ where
 /// This is a Value pointer to a RTypedData struct, Ruby’s internal
 /// representation of objects that wrap foreign types. Unlike [`RTypedData`] it
 /// tracks the Rust type it should contains and errors early in [`TryConvert`]
-/// if types don't match, rather than on [`Obj::get`].
+/// if types don't match.
 ///
 /// See the [`ReprValue`] and [`Object`] traits for additional methods
 /// available on this type. See [`Ruby`](Ruby#typed_dataobj) for methods to
@@ -698,6 +701,7 @@ where
     /// # Examples
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::{class, define_class, prelude::*, typed_data};
     /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
@@ -714,9 +718,10 @@ where
     /// # let _ = Point { x: 1, y: 2 }.x + Point { x: 3, y: 4 }.y;
     /// ```
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::obj_wrap` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn wrap(data: T) -> Self {
         get_ruby!().obj_wrap(data)
@@ -737,6 +742,7 @@ where
     /// # Examples
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::{class, define_class, prelude::*, typed_data};
     /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
@@ -762,6 +768,7 @@ where
     /// `new` method rather than `initialize`)
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::{
     ///     class, define_class, eval, function, method, prelude::*, typed_data, RClass, Value,
     /// };
@@ -800,22 +807,14 @@ where
     /// # let _ = Point { x: 1, y: 2 }.x + Point { x: 3, y: 4 }.y;
     /// ```
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::obj_wrap_as` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn wrap_as(data: T, class: RClass) -> Self {
         get_ruby!().obj_wrap_as(data, class)
     }
-
-    #[doc(hidden)]
-    #[deprecated(
-        since = "0.6.0",
-        note = "Obj::get() is unnecessary, Obj<T> derefs to T"
-    )]
-    pub fn get(&self) -> &T {
-        self.inner.get().unwrap()
-    }
 }
 
 impl<T> Deref for Obj<T>
@@ -829,8 +828,7 @@ where
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, define_class, typed_data};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, Ruby};
     ///
     /// #[magnus::wrap(class = "Point")]
     /// #[derive(Debug, PartialEq, Eq)]
@@ -839,13 +837,23 @@ where
     ///     y: isize,
     /// }
     ///
-    /// define_class("Point", class::object()).unwrap();
-    /// let value = typed_data::Obj::wrap(Point { x: 4, y: 2 });
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     ruby.define_class("Point", ruby.class_object())?;
+    ///     let value = ruby.obj_wrap(Point { x: 4, y: 2 });
     ///
-    /// assert_eq!(&*value, &Point { x: 4, y: 2 });
+    ///     assert_eq!(&*value, &Point { x: 4, y: 2 });
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn deref(&self) -> &Self::Target {
-        self.inner.get().unwrap()
+        // Since we've already validated the inner during `TryConvert` via `RTypedData::get`, we
+        // can skip the extra checks and libruby calls and just access the data directly.
+        unsafe {
+            let data_ptr = RTYPEDDATA_GET_DATA(self.inner.as_rb_value()) as *mut Self::Target;
+            &*data_ptr
+        }
     }
 }
 
@@ -919,6 +927,95 @@ where
     }
 }
 
+/// A trait for types that can be used with the `rb_gc_writebarrier` API.
+pub trait Writebarrier: ReprValue {
+    /// Inform Ruby that `self` contains a reference to `new`.
+    ///
+    /// If you have a Rust type that contains Ruby values, and itself is
+    /// wrapped as a Ruby object, and you choose to enable the `wb_protected`
+    /// flag so that it can participate in generational GC then all operations
+    /// that add a Ruby value to your data type must call this function.
+    ///
+    /// The write barrier is hard to get right. Magnus recommends you do not
+    /// enable the `wb_protected` flag, and thus you don't need to use this function.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use std::cell::RefCell;
+    ///
+    /// use magnus::{
+    ///     function, gc, method,
+    ///     prelude::*,
+    ///     typed_data::{Obj, Writebarrier},
+    ///     value::Opaque,
+    ///     DataTypeFunctions, Error, Ruby, TypedData, Value,
+    /// };
+    ///
+    /// #[derive(TypedData)]
+    /// #[magnus(class = "MyVec", free_immediately, mark, wb_protected)]
+    /// struct MyVec {
+    ///     values: RefCell<Vec<Opaque<Value>>>,
+    /// }
+    ///
+    /// impl DataTypeFunctions for MyVec {
+    ///     fn mark(&self, marker: &gc::Marker) {
+    ///         marker.mark_slice(self.values.borrow().as_slice());
+    ///     }
+    /// }
+    ///
+    /// impl MyVec {
+    ///     fn new() -> Self {
+    ///         Self {
+    ///             values: RefCell::new(Vec::new()),
+    ///         }
+    ///     }
+    ///
+    ///     fn push(ruby: &Ruby, rb_self: Obj<Self>, val: Value) -> Obj<Self> {
+    ///         rb_self.values.borrow_mut().push(val.into());
+    ///         rb_self.writebarrier(rb_self);
+    ///         rb_self
+    ///     }
+    /// }
+    ///
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let class = ruby.define_class("MyVec", ruby.class_object())?;
+    ///     class.define_singleton_method("new", function!(MyVec::new, 0))?;
+    ///     class.define_method("push", method!(MyVec::push, 1))?;
+    ///
+    ///     let _: Value = ruby.eval(
+    ///         r#"
+    ///             vec = MyVec.new
+    ///             vec.push("test")
+    ///             vec.push("example")
+    ///         "#,
+    ///     )?;
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap();
+    /// ```
+    fn writebarrier<T>(&self, young: T)
+    where
+        T: Mark,
+    {
+        unsafe { rb_gc_writebarrier(self.as_rb_value(), young.raw()) };
+    }
+
+    /// Opts `self` out of generational GC / write barrier protection.
+    ///
+    /// After calling this function `self` will not participate in generational
+    /// GC and will always be scanned during a GC.
+    /// See [`RTypedData::writebarrier`].
+    fn writebarrier_unprotect<T, U>(&self) {
+        unsafe { rb_gc_writebarrier_unprotect(self.as_rb_value()) };
+    }
+}
+
+impl Writebarrier for RTypedData {}
+
+impl<T> Writebarrier for Obj<T> where T: TypedData {}
+
 /// Trait for a Ruby-compatible `#hash` method.
 ///
 /// Automatically implemented for any type implementing [`std::hash::Hash`].
@@ -931,8 +1028,8 @@ where
 /// use std::hash::Hasher;
 ///
 /// use magnus::{
-///     class, define_class, function, gc, method, prelude::*, typed_data, value::Opaque,
-///     DataTypeFunctions, IntoValue, RHash, TypedData, Value,
+///     function, gc, method, prelude::*, typed_data, value::Opaque, DataTypeFunctions, Error,
+///     Ruby, TypedData, Value,
 /// };
 ///
 /// #[derive(TypedData)]
@@ -987,28 +1084,34 @@ where
 ///
 /// impl Eq for Pair {}
 ///
-/// # let _cleanup = unsafe { magnus::embed::init() };
-/// #
-/// let class = define_class("Pair", class::object()).unwrap();
-/// class
-///     .define_singleton_method("new", function!(Pair::new, 2))
-///     .unwrap();
-/// class
-///     .define_method("hash", method!(<Pair as typed_data::Hash>::hash, 0))
-///     .unwrap();
-/// class
-///     .define_method("eql?", method!(<Pair as typed_data::IsEql>::is_eql, 1))
-///     .unwrap();
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let class = ruby.define_class("Pair", ruby.class_object())?;
+///     class.define_singleton_method("new", function!(Pair::new, 2))?;
+///     class.define_method("hash", method!(<Pair as typed_data::Hash>::hash, 0))?;
+///     class.define_method("eql?", method!(<Pair as typed_data::IsEql>::is_eql, 1))?;
 ///
-/// let a = Pair::new("foo".into_value(), 1.into_value());
-/// let hash = RHash::new();
-/// hash.aset(a, "test value").unwrap();
+///     let a = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     let hash = ruby.hash_new();
+///     hash.aset(a, "test value")?;
 ///
-/// let b = Pair::new("foo".into_value(), 1.into_value());
-/// assert_eq!("test value", hash.fetch::<_, String>(b).unwrap());
+///     let b = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     assert_eq!("test value", hash.fetch::<_, String>(b)?);
 ///
-/// let c = Pair::new("bar".into_value(), 2.into_value());
-/// assert!(hash.get(c).is_none());
+///     let c = Pair::new(
+///         ruby.str_new("bar").as_value(),
+///         ruby.integer_from_i64(2).as_value(),
+///     );
+///     assert!(hash.get(c).is_none());
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub trait Hash {
     // Docs at trait level.
@@ -1043,8 +1146,8 @@ where
 /// use std::hash::Hasher;
 ///
 /// use magnus::{
-///     class, define_class, function, gc, method, prelude::*, typed_data, value::Opaque,
-///     DataTypeFunctions, IntoValue, RHash, TypedData, Value,
+///     function, gc, method, prelude::*, typed_data, value::Opaque, DataTypeFunctions, Error,
+///     Ruby, TypedData, Value,
 /// };
 ///
 /// #[derive(TypedData)]
@@ -1099,28 +1202,34 @@ where
 ///
 /// impl Eq for Pair {}
 ///
-/// # let _cleanup = unsafe { magnus::embed::init() };
-/// #
-/// let class = define_class("Pair", class::object()).unwrap();
-/// class
-///     .define_singleton_method("new", function!(Pair::new, 2))
-///     .unwrap();
-/// class
-///     .define_method("hash", method!(<Pair as typed_data::Hash>::hash, 0))
-///     .unwrap();
-/// class
-///     .define_method("eql?", method!(<Pair as typed_data::IsEql>::is_eql, 1))
-///     .unwrap();
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let class = ruby.define_class("Pair", ruby.class_object())?;
+///     class.define_singleton_method("new", function!(Pair::new, 2))?;
+///     class.define_method("hash", method!(<Pair as typed_data::Hash>::hash, 0))?;
+///     class.define_method("eql?", method!(<Pair as typed_data::IsEql>::is_eql, 1))?;
 ///
-/// let a = Pair::new("foo".into_value(), 1.into_value());
-/// let hash = RHash::new();
-/// hash.aset(a, "test value").unwrap();
+///     let a = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     let hash = ruby.hash_new();
+///     hash.aset(a, "test value")?;
 ///
-/// let b = Pair::new("foo".into_value(), 1.into_value());
-/// assert_eq!("test value", hash.fetch::<_, String>(b).unwrap());
+///     let b = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     assert_eq!("test value", hash.fetch::<_, String>(b)?);
 ///
-/// let c = Pair::new("bar".into_value(), 2.into_value());
-/// assert!(hash.get(c).is_none());
+///     let c = Pair::new(
+///         ruby.str_new("bar").as_value(),
+///         ruby.integer_from_i64(2).as_value(),
+///     );
+///     assert!(hash.get(c).is_none());
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub trait IsEql {
     // Docs at trait level.
@@ -1153,8 +1262,8 @@ where
 /// use std::cmp::Ordering;
 ///
 /// use magnus::{
-///     class, define_class, function, gc, method, module, prelude::*, rb_assert, typed_data,
-///     value::Opaque, DataTypeFunctions, IntoValue, Module, TypedData, Value,
+///     function, gc, method, prelude::*, rb_assert, typed_data, value::Opaque, DataTypeFunctions,
+///     Error, Module, Ruby, TypedData, Value,
 /// };
 ///
 /// #[derive(TypedData)]
@@ -1206,28 +1315,45 @@ where
 ///     }
 /// }
 ///
-/// # let _cleanup = unsafe { magnus::embed::init() };
-/// #
-/// let class = define_class("Pair", class::object()).unwrap();
-/// class
-///     .define_singleton_method("new", function!(Pair::new, 2))
-///     .unwrap();
-/// class
-///     .define_method("<=>", method!(<Pair as typed_data::Cmp>::cmp, 1))
-///     .unwrap();
-/// class.include_module(module::comparable()).unwrap();
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let class = ruby.define_class("Pair", ruby.class_object())?;
+///     class.define_singleton_method("new", function!(Pair::new, 2))?;
+///     class.define_method("<=>", method!(<Pair as typed_data::Cmp>::cmp, 1))?;
+///     class.include_module(ruby.module_comparable())?;
 ///
-/// let a = Pair::new("foo".into_value(), 1.into_value());
-/// let b = Pair::new("foo".into_value(), 2.into_value());
-/// rb_assert!("a < b", a, b);
+///     let a = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     let b = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(2).as_value(),
+///     );
+///     rb_assert!(ruby, "a < b", a, b);
 ///
-/// let b = Pair::new("foo".into_value(), 2.into_value());
-/// let c = Pair::new("bar".into_value(), 3.into_value());
-/// rb_assert!("b > c", b, c);
+///     let b = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(2).as_value(),
+///     );
+///     let c = Pair::new(
+///         ruby.str_new("bar").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     rb_assert!(ruby, "b > c", b, c);
 ///
-/// let a = Pair::new("foo".into_value(), 1.into_value());
-/// let b = Pair::new("foo".into_value(), 2.into_value());
-/// rb_assert!("(a <=> b) == -1", a, b);
+///     let a = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     let b = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(2).as_value(),
+///     );
+///     rb_assert!(ruby, "(a <=> b) == -1", a, b);
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub trait Cmp {
     // Docs at trait level.
@@ -1261,8 +1387,8 @@ where
 /// use std::fmt;
 ///
 /// use magnus::{
-///     class, define_class, function, gc, method, prelude::*, rb_assert, typed_data,
-///     value::Opaque, DataTypeFunctions, IntoValue, TypedData, Value,
+///     function, gc, method, prelude::*, rb_assert, typed_data, value::Opaque, DataTypeFunctions,
+///     Error, Ruby, TypedData, Value,
 /// };
 ///
 /// #[derive(TypedData)]
@@ -1299,21 +1425,23 @@ where
 ///     }
 /// }
 ///
-/// # let _cleanup = unsafe { magnus::embed::init() };
-/// #
-/// let class = define_class("Pair", class::object()).unwrap();
-/// class
-///     .define_singleton_method("new", function!(Pair::new, 2))
-///     .unwrap();
-/// class
-///     .define_method(
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let class = ruby.define_class("Pair", ruby.class_object())?;
+///     class.define_singleton_method("new", function!(Pair::new, 2))?;
+///     class.define_method(
 ///         "inspect",
 ///         method!(<Pair as typed_data::Inspect>::inspect, 0),
-///     )
-///     .unwrap();
+///     )?;
 ///
-/// let pair = Pair::new("foo".into_value(), 1.into_value());
-/// rb_assert!(r#"pair.inspect == "Pair { a: \"foo\", b: 1 }""#, pair);
+///     let pair = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     rb_assert!(ruby, r#"pair.inspect == "Pair { a: \"foo\", b: 1 }""#, pair);
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub trait Inspect {
     // Docs at trait level.
@@ -1341,8 +1469,8 @@ where
 ///
 /// ```
 /// use magnus::{
-///     class, define_class, function, gc, method, prelude::*, rb_assert, typed_data,
-///     value::Opaque, DataTypeFunctions, IntoValue, TypedData, Value,
+///     function, gc, method, prelude::*, rb_assert, typed_data, value::Opaque, DataTypeFunctions,
+///     Error, Ruby, TypedData, Value,
 /// };
 ///
 /// #[derive(TypedData, Clone)]
@@ -1368,21 +1496,21 @@ where
 ///     }
 /// }
 ///
-/// # let _cleanup = unsafe { magnus::embed::init() };
-/// #
-/// let class = define_class("Pair", class::object()).unwrap();
-/// class
-///     .define_singleton_method("new", function!(Pair::new, 2))
-///     .unwrap();
-/// class
-///     .define_method("dup", method!(<Pair as typed_data::Dup>::dup, 0))
-///     .unwrap();
-/// class
-///     .define_method("clone", method!(<Pair as typed_data::Dup>::clone, -1))
-///     .unwrap();
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let class = ruby.define_class("Pair", ruby.class_object())?;
+///     class.define_singleton_method("new", function!(Pair::new, 2))?;
+///     class.define_method("dup", method!(<Pair as typed_data::Dup>::dup, 0))?;
+///     class.define_method("clone", method!(<Pair as typed_data::Dup>::clone, -1))?;
 ///
-/// let a = Pair::new("foo".into_value(), 1.into_value());
-/// rb_assert!("b = a.dup; a.object_id != b.object_id", a);
+///     let a = Pair::new(
+///         ruby.str_new("foo").as_value(),
+///         ruby.integer_from_i64(1).as_value(),
+///     );
+///     rb_assert!(ruby, "b = a.dup; a.object_id != b.object_id", a);
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub trait Dup: Sized {
     // Docs at trait level.
