@@ -1,9 +1,11 @@
 //! Exposes JavaScript's reflection API to Rust.
 
-use crate::context::Context;
-use crate::handle::{Handle, Managed};
-use crate::result::JsResult;
-use crate::types::{build, JsString, JsValue};
+use crate::{
+    context::Context,
+    handle::Handle,
+    result::JsResult,
+    types::{build, private::ValueInternal, JsString, JsValue},
+};
 
 pub fn eval<'a, 'b, C: Context<'a>>(
     cx: &mut C,
@@ -11,6 +13,6 @@ pub fn eval<'a, 'b, C: Context<'a>>(
 ) -> JsResult<'a, JsValue> {
     let env = cx.env().to_raw();
     build(cx.env(), |out| unsafe {
-        neon_runtime::string::run_script(out, env, script.to_raw())
+        crate::sys::string::run_script(out, env, script.to_local())
     })
 }
